/***********************************************************
 * dirgraph.c -- member functions of class _DI_Graph       *
 * Jun Yang, November 17, 1994				   *
 ***********************************************************/

/*--------------- Header Files ---------------*/

#include "swan.h"
#include "laydir.h"

// Functions 

_DI_Graph::_DI_Graph(void) 
{
	node_list = new _DI_NODELIST ;
}

_DI_Graph::~_DI_Graph(void) 
{
	delete node_list ;
	node_list = NULL ;
}

_DI_NODELIST* _DI_Graph::getNodeList(void)
{
	return node_list ;
}

_DI_Node* _DI_Graph::searchNode(_DI_NODEID t_nodeid) 
{
	bool 	ok ;
	_DI_Node*	temp_node ;

	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    if ((*temp_node).getID() == t_nodeid)
		return temp_node ;
	    ok = (*node_list).getNextNode(temp_node) ;
	}
	return (_DI_Node*)NULL ;
}

bool _DI_Graph::insertNode(_DI_NODEID t_nodeid) 
{
	_DI_Node*	q = NULL ;

	if (searchNode(t_nodeid) != NULL)
	    return FALSE ;
	q = new _DI_Node(t_nodeid) ;
	(*node_list).insert(q) ;
	return TRUE ;
}

bool _DI_Graph::deleteNode(_DI_NODEID t_nodeid) 
{
	_DI_Node*	q = NULL ;

	if ((q = searchNode(t_nodeid)) != NULL) {
	    (*node_list).remove(q) ;
	    delete q ;
	    return TRUE ;
	}
	return FALSE ;
}

bool _DI_Graph::insertEdge(_DI_NODEID t_id1, _DI_NODEID t_id2) 
{
	_DI_Node*	p1 = NULL ;
	_DI_Node*	p2 = NULL ;
	_DI_nbnode*     q = NULL ;

	if ((p1 = searchNode(t_id1)) == NULL) 
	    return FALSE ;

	if ((p2 = searchNode(t_id2)) == NULL) 
	    return FALSE ;

	if ((q = searchNBNode(p1, p2)) != NULL)
	    return FALSE ;

	q = new _DI_nbnode ;
	q->id = p2 ;
	insertNBNode(p1, q) ;
	return TRUE ;
}

bool _DI_Graph::deleteEdge(_DI_NODEID t_id1, _DI_NODEID t_id2) 
{
	_DI_Node*	p1 = NULL ;
	_DI_Node*	p2 = NULL ;
	_DI_nbnode*     q = NULL ;

	if ((p1 = searchNode(t_id1)) == NULL) 
	    return FALSE ;

	if ((p2 = searchNode(t_id2)) == NULL) 
	    return FALSE ;

	if ((q = searchNBNode(p1, p2)) != NULL) {
	    removeNBNode(p1, q) ;
	    delete q ;
	    return TRUE ;
	}
	return FALSE ;
}

void _DI_Graph::insertNodeEdge(_DI_NODEID t_id1, _DI_NODEID t_id2) 
{
	insertNode(t_id1) ;
	insertNode(t_id2) ;
	insertEdge(t_id1, t_id2) ;
}

_DI_nbnode* _DI_Graph::searchNBNode(_DI_Node* t_node, _DI_Node* dest_id) 
{
	bool		ok ;
	_DI_nbnode*	q = NULL ;
	_DI_NBLIST*	temp_nblist ;

	temp_nblist = (*t_node).getNBList() ;
	ok = (*temp_nblist).getFirstNode(q) ;
	while (ok) {
	    if (q->id == dest_id)
		return q ;
	    ok = (*temp_nblist).getNextNode(q) ;
	}
	return (_DI_nbnode*)NULL ;
}

bool _DI_Graph::insertNBNode(_DI_Node* t_node, _DI_nbnode* t_nb) 
{
	(*((*t_node).getNBList())).insert(t_nb) ;
	return TRUE ;
}

bool _DI_Graph::removeNBNode(_DI_Node* t_node, _DI_nbnode* t_nb) 
{
	(*((*t_node).getNBList())).remove(t_nb) ;
	return TRUE ;
}

void _DI_Graph::markVisitedFlags(bool t_mark) 
{
	bool		ok ;
	_DI_Node*	temp_node ;

	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    (*temp_node).markVisited(t_mark) ;
	    ok = (*node_list).getNextNode(temp_node) ;
	}
}	
 
/***********************************************************
 * End of dirgraph.c    				   *
 ***********************************************************/
