/************************************************************
 * event.c --- the Swan event handler		  	    *
 * Jun Yang, November 17, 1994				    *
 ************************************************************/

/*---------- Header File ----------*/

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
extern "C" {
#include "geosim.h"
}
#include "swan.h"
#include <stdarg.h>
#define GS_MAIN_ARGS_A hInstance, hPrevInstance, lpszCmdParam, nCmdShow

/*--------------- Constants ---------------*/

#define INITIAL_ITEM	0
#define MAX_OUT_SEARCH_LN 14

/*--------------- Global Variables ---------------*/

BigString DumpFileName = "";
bool DumpSuccess = FALSE;
bool DumpResultsActive = FALSE;    
bool DumpWinActive = FALSE;
GraphicsFileType GraphFileFmt = PS_GRAY ; 

IVALUE _list_nodetype[] = {
	BOX,
	CIRCLE
} ;

IVALUE _list_nodefill[] = {
	FRAMED,
	FILLED
} ;

IVALUE _list_color[] = {
        BLACK,          
        DGRAY,          
        MGRAY,         
        LGRAY,        
        BLUEGRAY,       
        LBLUE,         
        PEACH,          
        LCYAN,         
        MCYAN,         
        GRAY,         
        MYELLOW,    
        LYELLOW,   
        MAGENTA,  
        DGREEN,  
        PASGREEN,      
} ;

IVALUE _list_lineth[] = {
	THINLINE,
	THICKLINE 
} ;

IVALUE _list_label[] = {
	ON,
	OFF
} ;

IVALUE _list_layout[] = {
	LISTACROSS,
	CIRCLENET,
	TREE,
	KKNET,
	HIERACHY
} ;

IVALUE _list_layoutmode[] = {
	AUTOLAYOUT,
	MANULAYOUT 
} ;

int fProgState = RUN ;
int fNextStep = 0 ;

bool fWaitRun = FALSE ;
bool fWaitStep = FALSE ;
bool fWaitQuit = FALSE ;
bool fWaitInsNode = FALSE ;
bool fWaitDelNode = FALSE ;
bool fWaitInsEdge = FALSE ;
bool fWaitDelEdge = FALSE ;
bool fWaitPosPicked = FALSE ;
bool fWaitGraphPicked = FALSE ;
bool fWaitNodePicked = FALSE ;
bool fWaitEdgePicked = FALSE ;
bool fWaitStrInput = FALSE ;
bool fWaitNodeChangeAttr = FALSE ;
bool fWaitEdgeChangeAttr = FALSE ;
bool fWaitGraphChangeAttr = FALSE ;
bool fWaitGlobalChangeAttr = FALSE ;
bool	fWaitGraphSave = FALSE ;
 
NODETYPE _gm_nodetype ;
COLORTYPE _gm_nodecolor ;
FLAG     _gm_nodefill ;
int      _gm_nodewidth ;
int      _gm_nodeheight ;
int      _gm_nodelineth ;
int      _gm_edgelength ;
int      _gm_edgelineth ;
COLORTYPE _gm_edgecolor ;
FLAG	 _gm_edgelabel ;
int	  _gm_layout ;
FLAG	  _gm_relayout ;
int	  _gm_layoutmode ;
int	  _gm_graphx ;
int	  _gm_graphy ;

NODETYPE _nm_nodetype ;
COLORTYPE _nm_nodecolor ;
FLAG     _nm_nodefill ;
int      _nm_nodewidth ;
int      _nm_nodeheight ;
int      _nm_nodelineth ;
int      _em_edgelineth ;
COLORTYPE _em_edgecolor ;

FLAG	_gl_autoredraw ;

int sc_x_origin = 0 ;
int sc_y_origin = 0 ;
float zoom_factor = 1.0 ;

// active named window
bool	fGraphAttrActive = FALSE ;
bool fNodeAttrActive = FALSE ;
bool fEdgeAttrActive = FALSE ;
bool fGlobalAttrActive = FALSE ;

int prev_node_color = INITIAL_ITEM ;
int prev_node_fill = INITIAL_ITEM ;
int prev_node_type = INITIAL_ITEM ;
int prev_node_lineth = INITIAL_ITEM ;
int prev_edge_color = INITIAL_ITEM ;
int prev_edge_lineth = INITIAL_ITEM ;
int prev_edge_label = INITIAL_ITEM ;
int prev_layout = INITIAL_ITEM ;
int prev_relayout = INITIAL_ITEM ;
int prev_layoutmode = INITIAL_ITEM ;
int prev_sn_color = INITIAL_ITEM ;
int prev_sn_fill = INITIAL_ITEM ;
int prev_sn_type = INITIAL_ITEM ;
int prev_sn_lineth = INITIAL_ITEM ;
int prev_se_color = INITIAL_ITEM ;
int prev_se_lineth = INITIAL_ITEM ;
int prev_gl_redraw = INITIAL_ITEM ;

char	 InputTextString[80];
char	 OutputTextString[80];

/*--------------- Function Prototypes --------------*/

extern "C" {
bool gfformat (void) ;
bool set_format (int) ;
bool confirmdump (void) ;
void highlightfield (BigString, BigString, GSColor, GSColor, GSColor) ;
bool canceldump (void) ;
bool dumpscreen(void) ;
bool run_anim();
bool step_anim();
bool move_left() ;
bool move_right() ;
bool move_up() ;
bool move_down() ;
bool zoom_in() ;
bool zoom_out() ;
bool do_swapnode() ;
bool do_movenode() ;
bool do_insnode() ;
bool do_delnode() ;
bool do_insedge() ;
bool do_deledge() ;
bool do_savegraph() ;
bool do_loadgraph() ;
bool do_pick(int, int, int) ;
bool do_remove(int, int, int) ;
bool do_text(int, int, char);

bool doquit();
bool quit();
bool do_nothing() ;
bool redraw() ;
void redraw_all() ;
void redraw_coord(int, int) ;
void erase_coord(void) ;
bool  char_is_legal(char ch) ;
int getItemIndex(INDEX, IVALUE) ;
void set_graph_button(FLAG) ;

bool change_graph_attrs() ;
bool change_node_attrs() ;
bool change_edge_attrs() ;
bool change_global_attrs() ;

bool set_node_type(char *) ;
bool set_node_fill(char *) ;
bool set_node_th(char *) ;
bool set_edge_th(char *) ;
bool set_edge_label(char *) ;
bool set_node_color(char *) ;
bool set_edge_color(char *) ;
bool set_layout(char *) ;
bool set_relayout(char *) ;
bool dec_gx() ;
bool inc_gx() ;
bool dec_gy() ;
bool inc_gy() ;
bool dec_nodewd() ;
bool inc_nodewd() ;
bool dec_nodeht() ;
bool inc_nodeht() ;
bool dec_edgelen() ;
bool inc_edgelen() ;
bool ok_change() ;
bool cancel_change() ;

bool set_sn_color(char *) ;
bool set_sn_type(char *) ;
bool set_sn_fill(char *) ;
bool set_sn_th(char *) ;
bool sn_decwd() ;
bool sn_incwd() ;
bool sn_decht() ;
bool sn_incht() ;
bool sn_ok_change() ;
bool sn_cancel_change() ;

bool set_se_th(char *) ;
bool set_se_color(char *) ;
bool se_ok_change() ;
bool se_cancel_change() ;

void show_gm_graphx() ;
void show_gm_graphy() ;
void show_gm_nodewidth() ;
void show_gm_nodeheight() ;
void show_gm_edgelength() ;
void show_nm_nodewidth() ;
void show_nm_nodeheight() ;

bool set_gl_redraw(char *) ;
bool gl_ok_change() ;
bool gl_cancel_change() ;

void redraw_graph_attr() ;
void redraw_node_attr() ;
void redraw_edge_attr() ;
void redraw_global_attr() ;
}

bool run_anim(void)
{
    	switch (fProgState) {
    	case RUN:		// do nothing
	    break ;

    	case STEP:		// change back to RUN mode
	    fProgState = RUN ;
	    showstepframe(FALSE) ;
	    showrunframe(TRUE) ;
	    break ;

    	default:
	    break ;
    	}	
	if (fWaitRun)
	    fWaitRun = FALSE ;
	GSsetcurrentnamedwindow(SWG_SHOWWIN) ;
        return TRUE ;
}

bool step_anim()
{
	switch (fProgState) {
	case RUN:
	    fProgState = STEP ;
	    showrunframe(FALSE) ;
	    showstepframe(TRUE) ;
	    break ;

	case STEP:
	    fNextStep = !fNextStep ;
	    break ;

	default:
	    break ;
	}
	if (fWaitStep)
	    fWaitStep = FALSE ;
	GSsetcurrentnamedwindow(SWG_SHOWWIN) ;
	return (TRUE) ;
}

bool move_left()
{
    int		old_x=sc_x_origin ;
    sc_x_origin -= SCREENXOFFSET ;
    if (sc_x_origin <= SCREENLEFTBORDER)
	sc_x_origin = SCREENLEFTBORDER ;
    if (old_x != sc_x_origin) {
        swg_pan(old_x-sc_x_origin, 0) ;
        redraw_all() ;
    }
    return TRUE ;
}

bool move_right()
{
    int		old_x=sc_x_origin ;
    sc_x_origin += SCREENXOFFSET ;
    if (sc_x_origin >= SCREENRIGHTBORDER) 
	sc_x_origin = SCREENRIGHTBORDER ;
    if (old_x != sc_x_origin) {
        swg_pan(old_x-sc_x_origin, 0) ;
    	redraw_all() ;
    }
    return TRUE ;
}

bool move_up()
{
    int		old_y=sc_y_origin ;

    sc_y_origin -= SCREENYOFFSET ;
    if (sc_y_origin <= SCREENTOPBORDER) 
	sc_y_origin = SCREENTOPBORDER ;
    if (old_y != sc_y_origin) {
    	swg_pan(0, old_y - sc_y_origin) ;
    	redraw_all() ;
    }
    return TRUE ;
}

bool move_down()
{
    int		old_y=sc_y_origin ;
    sc_y_origin += SCREENYOFFSET ;
    if (sc_y_origin >= SCREENBOTTOMBORDER)
	sc_y_origin = SCREENBOTTOMBORDER ;
    if (old_y != sc_y_origin) {
        swg_pan(0, old_y - sc_y_origin) ;
        redraw_all() ;
    }
    return TRUE ;
}

bool zoom_in()
{
    zoom_factor += ZOOMSTEP ;
    if (zoom_factor >= ZOOMUPPERBOUND)
	zoom_factor = ZOOMUPPERBOUND ;
    swg_zoom() ;
    redraw_all() ;
    return TRUE ;
}

bool zoom_out()
{
    zoom_factor -= ZOOMSTEP ;
    if (zoom_factor <= ZOOMLOWERBOUND) 
	zoom_factor = ZOOMLOWERBOUND ;
    swg_zoom() ;
    redraw_all() ;
    return TRUE ;
}

bool redraw()
{
	GSredrawnamedwindow("root") ;
	GSredrawnamedwindow(SWG_SHOWWIN) ;
	GSsetcurrentnamedwindow(SWG_SHOWWIN) ;
	redraw_all() ;
	return TRUE ;
}

bool doquit()
{
    GSquit(NULL);
    return TRUE;
}

bool quit()
{
    	return(TRUE);
}

bool do_nothing()
{
	return TRUE ;
}

int showrunframe(bool flag)
{
	GSColor 	lcolor ;

	GSsetcurrentnamedwindow("root") ;
	if (flag) {
	    lcolor = GSconvertcolor("PASGREEN") ;
	    GSsetcolor(lcolor) ;
	    GSsetlinesize(THICKLINE) ;
	    GSwdrawrect(556, 352, 58, 36) ;
	    GSsetlinesize(THINLINE) ;
	    GSdoeventsandreturn() ;
	}
	else {
	    lcolor = GSconvertcolor("BLUEGRAY") ;
	    GSsetcolor(lcolor) ;
	    GSsetlinesize(THICKLINE) ;
	    GSwdrawrect(556, 352, 58, 36) ;
	    GSsetlinesize(THINLINE) ;
	    GSdoeventsandreturn() ;
	}
	return TRUE ;
}

int showstepframe(bool flag)
{
	GSColor 	lcolor ;

	GSsetcurrentnamedwindow("root") ;
	if (flag) {
	    lcolor = GSconvertcolor("PASGREEN") ;
	    GSsetcolor(lcolor) ;
	    GSsetlinesize(THICKLINE) ;
	    GSwdrawrect(556, 392, 58, 36) ;
	    GSsetlinesize(THINLINE) ;
	    GSdoeventsandreturn() ;
	}
	else {
	    lcolor = GSconvertcolor("BLUEGRAY") ;
	    GSsetcolor(lcolor) ;
	    GSsetlinesize(THICKLINE) ;
	    GSwdrawrect(556, 392, 58, 36) ;
	    GSsetlinesize(THINLINE) ;
	    GSdoeventsandreturn() ;
	}
	return TRUE ;
}

bool do_swapnode()
{
	sw_print("Please pick two nodes in the same graph") ;
	_sws_swapnode = ON ;
	_sw_nodepicked = 0 ;
	return TRUE ;
}

bool do_movenode()
{
	_sws_movenode = ON ;
	_sw_nodepicked = 0 ;
	sw_print("Please pick a node") ;
	return TRUE ;
}

bool do_insnode()
{
	fWaitInsNode = FALSE ;
	return TRUE ;
}

bool do_delnode()
{
	fWaitDelNode = FALSE ;
	return TRUE ;
}

bool do_insedge()
{
	fWaitInsEdge = FALSE ;
	return TRUE ;
}

bool do_deledge()
{
        fWaitDelEdge = FALSE ;
        return TRUE ;
}

bool do_savegraph()
{
	sw_print("Please pick a graph:") ;
	fWaitGraphSave = TRUE ;
	return TRUE ;
}

bool do_loadgraph()
{
	char	temp_name[80] ;
	char	temp_str[80] ;

	sw_print("Please enter the graph file name:") ;
	sw_getstr(temp_name) ;
	if (swg_loadgraph(temp_name)) {
	    sprintf(temp_str, "Graph %s is loaded", temp_name) ;
	    sw_print(temp_str) ;
	    sw_displayallgraphs() ;
	}
	else {
	    sprintf(temp_str, "Error in loading graph %s!", temp_name) ;
	    sw_print(temp_str) ;
	}
	return TRUE ;
}

bool do_pick(int mouse_x, int mouse_y, int status)
{
	static BasicNode*	temp_node = NULL ;
	char		temp_name[80] ;
	char		temp_str[80] ;
/*
	int mouse_y ;
	int status ;
	va_list	ap ;
	
        va_start(ap, mouse_x) ;
	mouse_y = va_arg(ap, int) ;
	status = va_arg(ap, int) ;
	va_end(ap) ;
*/
	
	mouse_x += 2 ;
	mouse_y += 2 ;
   	switch (status) {
	case DRAG_FINAL:
	case MOUSE_CLICK:
	    if (_sws_swapnode) {
		if ((*_sw_root_lc).inThisLC(mouse_x, mouse_y, PICKNODE)) {
		    _sw_nodepicked++ ;
		    if (_sw_nodepicked == 2) {
			sw_print("Two nodes picked.") ; 
			if (swg_swapnode(temp_node, _sw_pick_node)) {
			    sw_print("Two nodes are swapped") ;
			    sw_displayallgraphs() ;
			}
			else {
			    sw_print("These two nodes cannot be \
swapped.\n") ;
			} 
			_sws_swapnode = OFF ;
			_sw_nodepicked = 0 ;
			temp_node = NULL ;
		        // redisplay graph here
		    }
		    else {
			temp_node = _sw_pick_node ;
		   	sw_print("Please pick another node") ;
		    }
		}
		return TRUE ;
	    }		    
	    if (_sws_movenode) {
		if ((*_sw_root_lc).inThisLC(mouse_x, mouse_y, PICKNODE))
		{
		    _sw_nodepicked = 1 ;
		    sw_print("Please specify a location") ;
		}
		else {
		    if (_sw_nodepicked == 1) {
			if (swg_movenode(_sw_pick_node, mouse_x, mouse_y))
			{
			sw_print("The node has been moved") ;
			sw_displayallgraphs() ;
			}
			else {
			    sw_print("Nodes in this graph cannot be\
 moved!") ;
			}
		        _sws_movenode = OFF ;
			_sw_nodepicked = 0 ;
		    }
		}
		return TRUE ;
	    } 

	// wait for a position picked
	    if (fWaitPosPicked) {
		_sw_pick_pos.x = (int) ((mouse_x+sc_x_origin)/
					  zoom_factor) ;
		_sw_pick_pos.y = (int) ((mouse_y+sc_y_origin)/
					  zoom_factor) ;
		fWaitPosPicked = FALSE ;
		return TRUE ;
	    }

	// wait for a graph picked
	    if (fWaitGraphPicked) {
		if (swg_pick(mouse_x, mouse_y, PICKGRAPH)) {
		    fWaitGraphPicked = FALSE ;
		}
		return TRUE ;
	    }	

	// wait for a node picked
	    if (fWaitNodePicked) {
		if (swg_pick(mouse_x, mouse_y, PICKNODE)) 
		    fWaitNodePicked = FALSE ;
		return TRUE ;
	    }
		
	// wait for an edge picked
	    if (fWaitEdgePicked) {
		if (swg_pick(mouse_x, mouse_y, PICKEDGE)) {
		    fWaitEdgePicked = FALSE ;
		}
		return TRUE ;
	    }	

	// wait for an graph picked
	    if (fWaitGraphChangeAttr) {
		if (swg_pick(mouse_x, mouse_y, PICKGRAPH)) {
		    fWaitGraphChangeAttr = FALSE ;
		}
		return TRUE ;
	    }	

	// wait for an graph picked
	    if (fWaitGraphSave) {
		if (swg_pick(mouse_x, mouse_y, PICKGRAPH)) {
		    fWaitGraphSave = FALSE ;
		    sw_print("Please specify a file name:") ;
		    sw_getstr(temp_name) ;
		    if (swg_savegraph(temp_name, _sw_pick_graph)) {
			sprintf(temp_str, "Graph saved in %s",
				 temp_name) ;
			sw_print(temp_str) ;
		    }
		    else {
			sw_print("Error in saving graph.") ;
		    }
		}
		return TRUE ;
	    }	

	// wait for an node picked
	    if (fWaitNodeChangeAttr) {
		if (swg_pick(mouse_x, mouse_y, PICKNODE)) {
		    fWaitNodeChangeAttr = FALSE ;
		}
		return TRUE ;
	    }	

	// wait for an edge picked
	    if (fWaitEdgeChangeAttr) {
		if (swg_pick(mouse_x, mouse_y, PICKEDGE)) {
		    fWaitEdgeChangeAttr = FALSE ;
		}
		return TRUE ;
	    }	

	// general picking operations
	    if (swg_pick(mouse_x, mouse_y, PICKNODE)) {
		sw_print("ONE node picked") ; 
	    }
	    else if (swg_pick(mouse_x, mouse_y, PICKEDGE)) {
                sw_print("ONE edge picked") ;
            }
	    else 
		sw_print("NOTHING picked") ;
	    break ;
	
	case MOUSE_INSIDE:
	    redraw_coord(mouse_x, mouse_y) ;
	    break ;

	case MOUSE_OUTSIDE:
	    erase_coord() ;
	    break ;

	default:
	    break ;
	}
	return TRUE ;
}
	
bool do_remove(int mouse_x, int mouse_y, int status)
{
	switch (status) {
	case DRAG_FINAL:
	case MOUSE_CLICK:
	    GSremovenamedwindow("nodeinfo") ;
	    GSremovenamedwindow("edgeinfo") ;
	    break ;
	default:
	    break ;
	}
	return TRUE ;
}


/*
  char_is_legal
    Returns TRUE if ch is a legal input character (A-Z, a-z, '.', '-', or ' ').
    Returns FALSE if ch is not a legal input character.
*/
bool char_is_legal (
  char  ch  /*--- character to check for legality ---*/
)
{
  return ((isalnum(ch)) || (ch == '.') || (ch == '-') || (ch == ' '));
}


/*
  do_text
    Called when a character is typed at the keyboard.
    Processes a character typed at the keyboard.
*/
bool do_text (
  int   mouse_x,    /*--- not used here, but required by event handler ---*/
  int   mouse_y,    /*--- not used here, but required by event handler ---*/
  char  ch          /*--- character typed at the keyboard ---*/
)
{
  int        text_x, text_y, text_wd, text_ht;
  GSColor    text_color;
  int        background_x, background_y, background_wd, background_ht;
  GSColor    background_color;
  int        length;
  BigString  current_window;
  BigString  out_string;
  int        i;

  /*--- Don't process character if alert window is up. ---*/
/*
  if (AlertShowing)
  {
    GSbell();
    return (FALSE);
  }
*/

  /*--- Don't process character unless "io" is the current window. ---*/
/*
  GSgetcurrentwindowname(current_window);
  length = strlen(current_window);
  for (i = 0; i < length; i++)
  {
    current_window[i] = tolower(current_window[i]);
  }
  if (strcmp(current_window, "io") != 0)
  {
    GSbell();
    return (FALSE);
  }
*/

  GSsetcurrentnamedwindow("root");
  GSgetfieldrect("MESSAGE", &text_x, &text_y, &text_wd, &text_ht, &text_color);
  if (ch == '\r') {
      if (fWaitStrInput) {
	  strcpy(_sw_pick_string, InputTextString) ;
	  InputTextString[0] = '\0' ;
	  fWaitStrInput = FALSE ;
	  return TRUE ;
      }
      return FALSE ;
  }

  if (char_is_legal(ch))
  {
    length = strlen(InputTextString);
    /*--- check length of string (in characters) in memory ---*/
    if (length < BIGSTRINGLEN)
    {
      /*--- copy string and add character to temporary buffer ---*/
      strcpy(out_string, InputTextString);
      out_string[length] = ch;
      out_string[length+1] = '\0';
      /*--- check length of string (in pixels) on screen ---*/
      if (GStextwidth(out_string) <= text_wd)
      {
        /*--- update permanent buffer if string still fits in field ---*/
        strcpy(InputTextString, out_string);
/*
        if (strlen(InputTextString) == 1)
        {
          GSenablemenuitem("funcs_menu", CLEAR_TEXT_MENU_ITEM);
        }
*/
      }
      else /*--- string will not fit in field with new character added ---*/
      {
        /*--- alert user that string would be too long ---*/
/*
        GSalert(-1, TRUE, "typedtoofar", "");
        show_fill_color(GSconvertcolor("mgray"));
*/
        /*--- disable typing until alert window is removed by user ---*/
/*
        AlertShowing = TRUE;
*/
	GSbell() ;
        return FALSE;
      }
    }
  }
  else if ((ch == '\b') && (strlen(InputTextString) > 0))
  { /*--- rubout last character ---*/
    InputTextString[strlen(InputTextString)-1] = '\0';
/*
    if (strlen(InputTextString) == 0) 
    {
      GSdisablemenuitem("funcs_menu", CLEAR_TEXT_MENU_ITEM);
    }
*/
    strcpy(out_string, InputTextString);
  }
  else
  {
    return FALSE;
  }
  /*--- if we get here, there string has been updated, so rewrite it ---*/
  // add_cursor_if_space(out_string, text_wd);
/*
  GSgetfieldrect("MESSAGE", &background_x, &background_y, &background_wd,
                 &background_ht, &background_color);
  GSsetcolor(background_color);
*/
  GSsetcolor(GSconvertcolor(color_table[DGRAY])) ;
  GSwfillrect(text_x, text_y+12, text_wd, text_ht-12);
  GSsetcolor(text_color);
/*
  GSsetfont(LARGE);
*/
  GSwwritemsg(text_x, text_y+24, out_string);
  return TRUE;
}

int getItemIndex(INDEX index, IVALUE val)
{
	register int i ;

	switch (index) {

	case GNODECOLOR:
	case GEDGECOLOR:
	    for (i = 0 ; i < sizeof(_list_color)/sizeof(IVALUE) ; i++)
		if (_list_color[i] == val)
		    return i ;
	    break ;

	case GNODETYPE:
	    for (i = 0 ; i < sizeof(_list_nodetype)/sizeof(IVALUE) ; i++)
		if (_list_nodetype[i] == val)
		    return i ;
	    break ;

	case GNODEFILLED:
	    for (i = 0 ; i < sizeof(_list_nodefill)/sizeof(IVALUE) ; i++)
		if (_list_nodefill[i] == val)
		    return i ;
	    break ;

	case GNODELINETH:
	case GEDGELINETH:
	    for (i = 0 ; i < sizeof(_list_lineth)/sizeof(IVALUE) ; i++)
		if (_list_lineth[i] == val)
		    return i ;
	    break ;

	case GEDGELABEL:
	case GAUTORELAYOUT:
	case GAUTOREDRAW:
	    for (i = 0 ; i < sizeof(_list_label)/sizeof(IVALUE) ; i++)
		if (_list_label[i] == val)
		    return i ;
	    break ;

	case GLAYOUTMODE:
	    for (i = 0 ; i < sizeof(_list_layoutmode)/sizeof(IVALUE) ; i++)
		if (_list_layoutmode[i] == val)
		    return i ;
	    break ;

	case GLAYOUTTYPE:
	    for (i = 0 ; i < sizeof(_list_layout)/sizeof(IVALUE) ; i++)
		if (_list_layout[i] == val)
		    return i ;
	    break ;

	default:
	    break ;
	}
	return -1 ;
}

bool change_graph_attrs()
{
	int	text_x, text_y,  text_w, text_h ;
	GSColor text_color ;
	char	temp_str[80] ;
	int	t_id, i ;

	sw_print("Please pick a graph...") ;
	GSsetcurrentnamedwindow(SWG_SHOWWIN) ;
	fWaitGraphChangeAttr = TRUE ;
	while (fWaitGraphChangeAttr) 
	    GSdoeventsandreturn() ;

	// set internal variables to be modified
	_gm_nodetype = (*_sw_pick_graph).getNodeType() ;
	_gm_nodecolor = (*_sw_pick_graph).getNodeColor() ;
	_gm_nodefill = (*_sw_pick_graph).getNodeFilled() ;
	_gm_nodewidth = (*_sw_pick_graph).getNodeWidth() ;
	_gm_nodeheight = (*_sw_pick_graph).getNodeHeight() ;
	_gm_nodelineth = (*_sw_pick_graph).getNodeLineTH() ;
	_gm_edgelength = (*_sw_pick_graph).getEdgeLength() ;
	_gm_edgelineth = (*_sw_pick_graph).getEdgeLineTH() ;
	_gm_edgecolor = (*_sw_pick_graph).getEdgeColor() ;
	_gm_edgelabel = ((*_sw_pick_graph).getGraphInfo())->fShowLabel
;
	_gm_layout = (*_sw_pick_graph).getLayoutType() ;
	_gm_relayout = (*_sw_pick_graph).isAutoRelayout() ;
	_gm_layoutmode = (*_sw_pick_graph).getLayoutMode() ;
	_gm_graphx = ((*_sw_pick_graph).getPPos()).x ;
	_gm_graphy = ((*_sw_pick_graph).getPPos()).y ;

  	GSsetactivenamedwindow("GraphAttr");
	fGraphAttrActive = TRUE ;
        GSdrawnamedwindow("GraphAttr", BSTORE, POPUP) ;
	redraw_graph_attr() ;
	if ((t_id = getItemIndex(GNODECOLOR, _gm_nodecolor)) != -1) {
	    prev_node_color = t_id ;
	    GStogglecheck("col_menu", GSgetmenuitemname("col_menu", t_id)) ;
	}
	if ((t_id = getItemIndex(GNODETYPE, _gm_nodetype)) != -1) {
	    prev_node_type = t_id ;
	    GStogglecheck("nd_type_menu", GSgetmenuitemname("nd_type_menu", t_id)) ;
	}
	if ((t_id = getItemIndex(GNODEFILLED, _gm_nodefill)) != -1) {
	    prev_node_fill = t_id ;
	    GStogglecheck("nd_fill_menu", GSgetmenuitemname("nd_fill_menu", t_id)) ;
	}
	if ((t_id = getItemIndex(GNODELINETH, _gm_nodelineth)) != -1) {
	    prev_node_lineth = t_id ;
	    GStogglecheck("ndth_menu", GSgetmenuitemname("ndth_menu", t_id)) ;
	}
	if ((t_id = getItemIndex(GEDGECOLOR, _gm_edgecolor)) != -1) {
	    prev_edge_color = t_id ;
	    GStogglecheck("edcol_menu", GSgetmenuitemname("edcol_menu", t_id)) ;
	}
	if ((t_id = getItemIndex(GEDGELINETH, _gm_edgelineth)) != -1) {
	    prev_edge_lineth = t_id ;
	    GStogglecheck("edth_menu", GSgetmenuitemname("edth_menu", t_id)) ;
	}
	if ((t_id = getItemIndex(GEDGELABEL, _gm_edgelabel)) != -1) {
	    prev_edge_label = t_id ;
	    GStogglecheck("edlab_menu", GSgetmenuitemname("edlab_menu", t_id)) ;
	}
	if ((t_id = getItemIndex(GAUTORELAYOUT, _gm_relayout)) != -1) {
	    prev_relayout = t_id ;
	    GStogglecheck("relayout_menu", GSgetmenuitemname("relayout_menu", t_id)) ;
	}
	if ((t_id = getItemIndex(GLAYOUTMODE, _gm_layoutmode)) != -1) {
	    prev_layoutmode = t_id ;
	    GStogglecheck("laymode_menu", GSgetmenuitemname("laymode_menu", t_id)) ;
	    if (_gm_layoutmode == AUTOLAYOUT) {
		GSdisablebutton("GraphAttr", "incgx", DRAW) ;
		GSdisablebutton("GraphAttr", "decgx", DRAW) ;
		GSdisablebutton("GraphAttr", "incgy", DRAW) ;
		GSdisablebutton("GraphAttr", "decgy", DRAW) ;
	    }
	    else {
		GSenablebutton("GraphAttr", "incgx", DRAW) ;
		GSenablebutton("GraphAttr", "decgx", DRAW) ;
		GSenablebutton("GraphAttr", "incgy", DRAW) ;
		GSenablebutton("GraphAttr", "decgy", DRAW) ;
	    }
	}
	if ((t_id = getItemIndex(GLAYOUTTYPE, _gm_layout)) != -1) {
	    prev_layout = t_id ;
	    GStogglecheck("layout_menu", GSgetmenuitemname("layout_menu", t_id)) ;
	    switch (_gm_layout) {
	    case LISTACROSS:
		GSdisablemenuitem("layout_menu", "CircleNet") ;
		GSdisablemenuitem("layout_menu", "RootedTree") ;
		GSdisablemenuitem("layout_menu", "KK_Net") ;
		GSdisablemenuitem("layout_menu", "Hierarchy") ;
		break ;

	    case CIRCLENET:
		GSdisablemenuitem("layout_menu", "List") ;
		GSdisablemenuitem("layout_menu", "RootedTree") ;
		GSdisablemenuitem("layout_menu", "Hierarchy") ;
		break ;

	    case TREE:
		GSdisablemenuitem("layout_menu", "List") ;
		GSdisablemenuitem("layout_menu", "CircleNet") ;
		GSdisablemenuitem("layout_menu", "KK_Net") ;
		GSdisablemenuitem("layout_menu", "Hierarchy") ;
		break ;

	    case KKNET:
		GSdisablemenuitem("layout_menu", "List") ;
		GSdisablemenuitem("layout_menu", "RootedTree") ;
		GSdisablemenuitem("layout_menu", "Hierarchy") ;
		break ;

	    case HIERACHY:
		GSdisablemenuitem("layout_menu", "List") ;
		GSdisablemenuitem("layout_menu", "CircleNet") ;
		GSdisablemenuitem("layout_menu", "RootedTree") ;
		GSdisablemenuitem("layout_menu", "KK_Net") ;
		break ;

	    default:
		GSdisablemenuitem("layout_menu", "List") ;
		GSdisablemenuitem("layout_menu", "CircleNet") ;
		GSdisablemenuitem("layout_menu", "RootedTree") ;
		GSdisablemenuitem("layout_menu", "KK_Net") ;
		GSdisablemenuitem("layout_menu", "Hierarchy") ;
		break ;
	    }
	}
	GSdoeventsandreturn() ;
	return TRUE ;
}

void redraw_graph_attr()
{
	show_gm_graphx() ;
	show_gm_graphy() ;
	show_gm_nodewidth() ;
	show_gm_nodeheight() ;
	show_gm_edgelength() ;
	GSdoeventsandreturn() ;
}

bool change_node_attrs() 
{
	int	t_id ;

	sw_print("Please pick a node...") ;
	GSsetcurrentnamedwindow(SWG_SHOWWIN) ;
	fWaitNodeChangeAttr = TRUE ;
	while (fWaitNodeChangeAttr) 
	    GSdoeventsandreturn() ;

	// set internal variables to be modified
	_nm_nodetype = (*_sw_pick_node).getType() ;
	_nm_nodecolor = (*_sw_pick_node).getColor() ;
	_nm_nodefill = (*_sw_pick_node).getFilled() ;
	_nm_nodewidth = (*_sw_pick_node).getWidth() ;
	_nm_nodeheight = (*_sw_pick_node).getHeight() ;
	_nm_nodelineth = (*_sw_pick_node).getLineTH() ;

	GSsetactivenamedwindow("NodeAttr") ;
	fNodeAttrActive = TRUE ;
        GSdrawnamedwindow("NodeAttr", BSTORE, POPUP) ;
	redraw_node_attr() ;
	if ((t_id = getItemIndex(GNODECOLOR, _nm_nodecolor)) != -1) {
	    prev_sn_color = t_id ;
	    GStogglecheck("sn_col_menu", GSgetmenuitemname("sn_col_menu", t_id)) ;
	}
	if ((t_id = getItemIndex(GNODETYPE, _nm_nodetype)) != -1) {
	    prev_sn_type = t_id ;
	    GStogglecheck("sn_type_menu", GSgetmenuitemname("sn_type_menu", t_id)) ;
	}
	if ((t_id = getItemIndex(GNODEFILLED, _nm_nodefill)) != -1) {
	    prev_sn_fill = t_id ;
	    GStogglecheck("sn_fill_menu", GSgetmenuitemname("sn_fill_menu", t_id)) ;
	}
	if ((t_id = getItemIndex(GNODELINETH, _nm_nodelineth)) != -1) {
	    prev_sn_lineth = t_id ;
	    GStogglecheck("sn_th_menu", GSgetmenuitemname("sn_th_menu", t_id)) ;
	}
	return TRUE ;
}

void redraw_node_attr()
{
	show_nm_nodewidth() ;
	show_nm_nodeheight() ;
	GSdoeventsandreturn() ;
}
	
bool change_edge_attrs() 
{
	int 	t_id ;

	sw_print("Please pick an edge...") ;
	GSsetcurrentnamedwindow(SWG_SHOWWIN) ;
	fWaitEdgeChangeAttr = TRUE ;
	while (fWaitEdgeChangeAttr)
	    GSdoeventsandreturn() ;

	// set internal variables to be modified
	_em_edgecolor = (*_sw_pick_edge).getColor() ;
	_em_edgelineth = (*_sw_pick_edge).getLineTH() ;

	GSsetactivenamedwindow("EdgeAttr") ;
	fEdgeAttrActive = TRUE ;
        GSdrawnamedwindow("EdgeAttr", BSTORE, POPUP) ;

	redraw_edge_attr() ;
	if ((t_id = getItemIndex(GEDGECOLOR, _em_edgecolor)) != -1) {
	    prev_se_color = t_id ;
	    GStogglecheck("se_col_menu", GSgetmenuitemname("se_col_menu", t_id)) ;
	}
	if ((t_id = getItemIndex(GEDGELINETH, _em_edgelineth)) != -1) {
	    prev_se_lineth = t_id ;
	    GStogglecheck("se_th_menu", GSgetmenuitemname("se_th_menu", t_id)) ;
	}
	return TRUE ;
}

bool change_global_attrs() 
{
	int 	t_id ;

	GSsetcurrentnamedwindow(SWG_SHOWWIN) ;

	// set internal variables to be modified
	_gl_autoredraw = _sws_autoredraw ;

	GSsetactivenamedwindow("GlobalAttr") ;
	fGlobalAttrActive = TRUE ;
        GSdrawnamedwindow("GlobalAttr", BSTORE, POPUP) ;
	redraw_global_attr() ;

	if ((t_id = getItemIndex(GAUTOREDRAW, _gl_autoredraw)) != -1) {
	    prev_gl_redraw = t_id ;
	    GStogglecheck("gl_redraw_menu", GSgetmenuitemname("gl_redraw_menu", t_id)) ;
	}
	return TRUE ;
}

void redraw_global_attr()
{
	GSdoeventsandreturn() ;
}

bool set_gl_redraw(char *item_name)
{
  int item_num = GSgetmenuitemnum("gl_redraw_menu", item_name);
	GStogglecheck("gl_redraw_menu", GSgetmenuitemname("gl_redraw_menu", prev_gl_redraw));
	if (prev_gl_redraw != item_num) {
	    prev_gl_redraw = item_num ;
	    _gl_autoredraw = _list_label[item_num] ;
	}
	return TRUE ;
}

bool gl_ok_change()
{
	// set modified variables back
	if (_gl_autoredraw != _sws_autoredraw) {
	    _sws_autoredraw = _gl_autoredraw ;
	}
	GStogglecheck("gl_redraw_menu", GSgetmenuitemname("gl_redraw_menu", prev_gl_redraw)) ;

        GSremovenamedwindow("GlobalAttr") ;
  	GSsetdeactivenamedwindow("GlobalAttr");
	fGlobalAttrActive = FALSE ;
	GSsetcurrentnamedwindow(SWG_SHOWWIN) ;
	return TRUE ;
}
	
bool gl_cancel_change()
{
	GStogglecheck("gl_redraw_menu", GSgetmenuitemname("gl_redraw_menu", prev_gl_redraw)) ;
	GSremovenamedwindow("GlobalAttr") ;
  	GSsetdeactivenamedwindow("GlobalAttr");
	fGlobalAttrActive = FALSE ;
	return TRUE ;
}

void redraw_edge_attr()
{
	GSdoeventsandreturn() ;
}

bool set_node_type(char *item_name)
{
  int item_num = GSgetmenuitemnum("nd_type_menu", item_name);
	GStogglecheck("nd_type_menu", GSgetmenuitemname("nd_type_menu", prev_node_type));
	if (prev_node_type != item_num) {
	prev_node_type = item_num ;
	_gm_nodetype = _list_nodetype[item_num] ;
	}
	return TRUE ;
}

bool set_node_fill(char *item_name)
{
  int item_num = GSgetmenuitemnum("nd_fill_menu", item_name);
	GStogglecheck("nd_fill_menu", GSgetmenuitemname("nd_fill_menu", prev_node_fill));
	if (prev_node_fill != item_num) {
	prev_node_fill = item_num ;
	_gm_nodefill = _list_nodefill[item_num] ;
	}
	return TRUE ;
}

bool set_node_th(char *item_name)
{
  int item_num = GSgetmenuitemnum("ndth_menu", item_name);
	GStogglecheck("ndth_menu", GSgetmenuitemname("ndth_menu", prev_node_lineth));
	if (prev_node_lineth != item_num) {
	prev_node_lineth = item_num ;
	_gm_nodelineth = _list_lineth[item_num] ;
	}
	return TRUE ;
}

bool set_node_color(char *item_name)
{
  int item_num = GSgetmenuitemnum("col_menu", item_name);
	GStogglecheck("col_menu", GSgetmenuitemname("col_menu", prev_node_color));
	if (prev_node_color != item_num) {
	prev_node_color = item_num ;
	_gm_nodecolor = _list_color[item_num] ;
	}
	return TRUE ;
}

bool set_edge_th(char *item_name)
{
  int item_num = GSgetmenuitemnum("edth_menu", item_name);
	GStogglecheck("edth_menu", GSgetmenuitemname("edth_menu", prev_edge_lineth));
	if (prev_edge_lineth != item_num) {
	prev_edge_lineth = item_num ;
	_gm_edgelineth = _list_lineth[item_num] ;
	}
	return TRUE ;
}

bool set_edge_label(char *item_name)
{
  int item_num = GSgetmenuitemnum("edlab_menu", item_name);
	GStogglecheck("edlab_menu", GSgetmenuitemname("edlab_menu", prev_edge_label));
	if (prev_edge_label != item_num) {
	prev_edge_label = item_num ;
	_gm_edgelabel = _list_label[item_num] ;
	}
	return TRUE ;
}

bool set_edge_color(char *item_name)
{
  int item_num = GSgetmenuitemnum("edcol_menu", item_name);
	GStogglecheck("edcol_menu", GSgetmenuitemname("edcol_menu", prev_edge_color)) ;
	if (prev_edge_color != item_num) {
	prev_edge_color = item_num ;
	_gm_edgecolor = _list_color[item_num] ;
	}
	return TRUE ;
}

bool set_layout(char *item_name)
{
  int item_num = GSgetmenuitemnum("layout_menu", item_name);
	GStogglecheck("layout_menu", GSgetmenuitemname("layout_menu", prev_layout)) ;
	if (prev_layout != item_num) {
	    prev_layout = item_num ;
	    _gm_layout = _list_layout[item_num] ;
	}
	return TRUE ;
}

bool set_relayout(char *item_name)
{
  int item_num = GSgetmenuitemnum("relayout_menu", item_name);
	GStogglecheck("relayout_menu", GSgetmenuitemname("relayout_menu", prev_relayout)) ;
	if (prev_relayout != item_num) {
	    prev_relayout = item_num ;
	    _gm_relayout = _list_label[item_num] ;
	}
	return TRUE ;
}

bool set_layoutmode(char *item_name)
{
  int item_num = GSgetmenuitemnum("laymode_menu", item_name);
	GStogglecheck("laymode_menu", GSgetmenuitemname("laymode_menu", prev_layoutmode)) ;
	if (prev_layoutmode != item_num) {
	    prev_layoutmode = item_num ;
	    _gm_layoutmode = _list_layoutmode[item_num] ;
	    if (_gm_layoutmode == AUTOLAYOUT) {
		GSdisablebutton("GraphAttr", "incgx", DRAW) ;
		GSdisablebutton("GraphAttr", "decgx", DRAW) ;
		GSdisablebutton("GraphAttr", "incgy", DRAW) ;
		GSdisablebutton("GraphAttr", "decgy", DRAW) ;
	    }
	    else {
		GSenablebutton("GraphAttr", "incgx", DRAW) ;
		GSenablebutton("GraphAttr", "decgx", DRAW) ;
		GSenablebutton("GraphAttr", "incgy", DRAW) ;
		GSenablebutton("GraphAttr", "decgy", DRAW) ;
	    }
	}
	return TRUE ;
}

void show_gm_graphx()
{
	int	text_x, text_y, text_w, text_h ;
	GSColor	text_color ;
	char	temp_str[80] ;

	GSsetcurrentnamedwindow("GraphAttr") ;
	GSgetfieldrect("graphx", &text_x, &text_y, &text_w, &text_h,
			&text_color) ;
	GSsetcolor(GSconvertcolor("white")) ;
	GSwfillrect(text_x, text_y, text_w, text_h) ;
	GSsetcolor(text_color) ;
	sprintf(temp_str, "%d", _gm_graphx) ;
	GSwwritemsg(text_x, text_y+10, temp_str) ;
}

void show_gm_graphy()
{
	int	text_x, text_y, text_w, text_h ;
	GSColor	text_color ;
	char	temp_str[80] ;

	GSsetcurrentnamedwindow("GraphAttr") ;
	GSgetfieldrect("graphy", &text_x, &text_y, &text_w, &text_h,
			&text_color) ;
	GSsetcolor(GSconvertcolor("white")) ;
	GSwfillrect(text_x, text_y, text_w, text_h) ;
	GSsetcolor(text_color) ;
	sprintf(temp_str, "%d", _gm_graphy) ;
	GSwwritemsg(text_x, text_y+10, temp_str) ;
}

void show_gm_nodewidth()
{
	int	text_x, text_y, text_w, text_h ;
	GSColor	text_color ;
	char	temp_str[80] ;

	GSsetcurrentnamedwindow("GraphAttr") ;
	GSgetfieldrect("nodewidth", &text_x, &text_y, &text_w, &text_h,
			&text_color) ;
	GSsetcolor(GSconvertcolor("white")) ;
	GSwfillrect(text_x, text_y, text_w, text_h) ;
	GSsetcolor(text_color) ;
	sprintf(temp_str, "%d", _gm_nodewidth) ;
	GSwwritemsg(text_x, text_y+10, temp_str) ;
}

void show_gm_nodeheight()
{
	int	text_x, text_y, text_w, text_h ;
	GSColor	text_color ;
	char	temp_str[80] ;

	GSsetcurrentnamedwindow("GraphAttr") ;
	GSgetfieldrect("nodeheight", &text_x, &text_y, &text_w, &text_h,
			&text_color) ;
	GSsetcolor(GSconvertcolor("white")) ;
	GSwfillrect(text_x, text_y, text_w, text_h) ;
	GSsetcolor(text_color) ;
	sprintf(temp_str, "%d", _gm_nodeheight) ;
	GSwwritemsg(text_x, text_y+10, temp_str) ;
}

void show_gm_edgelength()
{
	int	text_x, text_y, text_w, text_h ;
	GSColor	text_color ;
	char	temp_str[80] ;

	GSsetcurrentnamedwindow("GraphAttr") ;
	GSgetfieldrect("edgelength", &text_x, &text_y, &text_w, &text_h,
			&text_color) ;
	GSsetcolor(GSconvertcolor("white")) ;
	GSwfillrect(text_x, text_y, text_w, text_h) ;
	GSsetcolor(text_color) ;
	sprintf(temp_str, "%d", _gm_edgelength) ;
	GSwwritemsg(text_x, text_y+10, temp_str) ;
}

bool dec_nodewd()
{
	if (_gm_nodewidth > _SW_MINNODEWIDTH)
	    _gm_nodewidth-- ;
	show_gm_nodewidth() ;
	return TRUE ;
}
	
bool inc_gx()
{
	if (_gm_graphx < SCREENRIGHTBORDER)
	    _gm_graphx++ ;
	show_gm_graphx() ;
	return TRUE ;
}
	
bool dec_gx()
{
	if (_gm_graphx > SCREENLEFTBORDER)
	    _gm_graphx-- ;
	show_gm_graphx() ;
	return TRUE ;
}

bool inc_gy()
{
	if (_gm_graphy < SCREENBOTTOMBORDER)
	    _gm_graphy++ ;
	show_gm_graphy() ;
	return TRUE ;
}
	
bool dec_gy()
{
	if (_gm_graphy > SCREENTOPBORDER)
	    _gm_graphy-- ;
	show_gm_graphy() ;
	return TRUE ;
}

bool inc_nodewd()
{
	if (_gm_nodewidth < _SW_MAXNODEWIDTH)
	    _gm_nodewidth++ ;
	show_gm_nodewidth() ;
	return TRUE ;
}
	
bool dec_nodeht()
{
	if (_gm_nodeheight > _SW_MINNODEHEIGHT)
	    _gm_nodeheight-- ;
	show_gm_nodeheight() ;
	return TRUE ;
}
	
bool inc_nodeht()
{
	if (_gm_nodeheight < _SW_MAXNODEHEIGHT)
	    _gm_nodeheight++ ;
	show_gm_nodeheight() ;
	return TRUE ;
}
	
bool dec_edgelen()
{
	if (_gm_edgelength > _SW_MINEDGELENGTH)
	    _gm_edgelength-- ;
	show_gm_edgelength() ;
	return TRUE ;
}
	
bool inc_edgelen()
{
	if (_gm_edgelength < _SW_MAXEDGELENGTH)
	    _gm_edgelength++ ;
	show_gm_edgelength() ;
	return TRUE ;
}

bool ok_change()
{
	bool		fChanged = FALSE ;
	LOCATION	temp_pos ;

	// set modified variables back
	if (_gm_nodetype != (*_sw_pick_graph).getNodeType()) {
	    (*_sw_pick_graph).setNodeType(_gm_nodetype) ;
	    fChanged = TRUE ;
	    (*_sw_pick_graph).setNewLayoutFlag(TRUE) ;
	}
	if (_gm_nodecolor != (*_sw_pick_graph).getNodeColor()) {
	    (*_sw_pick_graph).setNodeColor(_gm_nodecolor) ;
	    fChanged = TRUE ;
	}
	if (_gm_nodefill != (*_sw_pick_graph).getNodeFilled()) {
	    (*_sw_pick_graph).setNodeFilled(_gm_nodefill) ; 
	    fChanged = TRUE ;
	}
	if (_gm_nodewidth != (*_sw_pick_graph).getNodeWidth()) {
	    (*_sw_pick_graph).setNodeWidth(_gm_nodewidth) ; 
	    fChanged = TRUE ;
	    (*_sw_pick_graph).setNewLayoutFlag(TRUE) ;
	}
	if (_gm_nodeheight != (*_sw_pick_graph).getNodeHeight()) {
	    (*_sw_pick_graph).setNodeHeight(_gm_nodeheight) ;     
	    fChanged = TRUE ;
	    (*_sw_pick_graph).setNewLayoutFlag(TRUE) ;
	}
	if (_gm_nodelineth != (*_sw_pick_graph).getNodeLineTH()) {
	    (*_sw_pick_graph).setNodeLineTH(_gm_nodelineth) ; 
	    fChanged = TRUE ;
	}
	if (_gm_edgelength != (*_sw_pick_graph).getEdgeLength()) {
	    (*_sw_pick_graph).setEdgeLineTH(_gm_edgelineth) ;
	    fChanged = TRUE ;
	    (*_sw_pick_graph).setNewLayoutFlag(TRUE) ;
	}
	if (_gm_edgelineth != (*_sw_pick_graph).getEdgeLineTH()) {
	    (*_sw_pick_graph).setEdgeLineTH(_gm_edgelineth) ;
	    fChanged = TRUE ;
	}
	if (_gm_edgelabel != ((*_sw_pick_graph).getGraphInfo())->fShowLabel) {
	    ((*_sw_pick_graph).getGraphInfo())->fShowLabel =
							_gm_edgelabel ;
	    fChanged = TRUE ;
	    (*_sw_pick_graph).setNewLayoutFlag(TRUE) ;
	}
	if (_gm_edgecolor != (*_sw_pick_graph).getEdgeColor()) {
	    (*_sw_pick_graph).setEdgeColor(_gm_edgecolor) ;
	    fChanged = TRUE ;
	}
	if (_gm_layout != (*_sw_pick_graph).getLayoutType()) {
	    (*_sw_pick_graph).setLayoutType(_gm_layout) ;
	    fChanged = TRUE ;
	    (*_sw_pick_graph).setNewLayoutFlag(TRUE) ;
	}
	if (_gm_relayout != (*_sw_pick_graph).isAutoRelayout()) {
	    (*_sw_pick_graph).setAutoRelayout(_gm_relayout) ;
	    fChanged = TRUE ;
	}
	if (_gm_layoutmode != (*_sw_pick_graph).getLayoutMode()) {
	    (*_sw_pick_graph).setLayoutMode(_gm_layoutmode) ;
	    fChanged = TRUE ;
	}
	temp_pos = (*_sw_pick_graph).getPPos() ;
	if (_gm_graphx != temp_pos.x || _gm_graphy != temp_pos.y) {
	    temp_pos.x = _gm_graphx ;
	    temp_pos.y = _gm_graphy ;
	    (*_sw_pick_graph).setPPos(temp_pos) ;
	    fChanged = TRUE ;
	}

	GStogglecheck("col_menu", GSgetmenuitemname("col_menu", prev_node_color)) ;
	GStogglecheck("nd_type_menu", GSgetmenuitemname("nd_type_menu", prev_node_type)) ;
	GStogglecheck("nd_fill_menu", GSgetmenuitemname("nd_fill_menu", prev_node_fill)) ;
	GStogglecheck("ndth_menu", GSgetmenuitemname("ndth_menu", prev_node_lineth)) ;
	GStogglecheck("edth_menu", GSgetmenuitemname("edth_menu", prev_edge_lineth)) ;
	GStogglecheck("edlab_menu", GSgetmenuitemname("edlab_menu", prev_edge_label)) ;
	GStogglecheck("edcol_menu", GSgetmenuitemname("edcol_menu", prev_edge_color)) ;
	GStogglecheck("layout_menu", GSgetmenuitemname("layout_menu", prev_layout)) ;
	GStogglecheck("relayout_menu", GSgetmenuitemname("relayout_menu", prev_relayout)) ;
	GStogglecheck("laymode_menu", GSgetmenuitemname("laymode_menu", prev_layoutmode)) ;
        GSenablemenuitem("layout_menu", "List") ;
        GSenablemenuitem("layout_menu", "CircleNet") ;
        GSenablemenuitem("layout_menu", "RootedTree") ;
        GSenablemenuitem("layout_menu", "KK_Net") ;
        GSenablemenuitem("layout_menu", "Hierarchy") ;

        GSremovenamedwindow("GraphAttr") ;
  	GSsetdeactivenamedwindow("GraphAttr");
	fGraphAttrActive = FALSE ;
	GSsetcurrentnamedwindow(SWG_SHOWWIN) ;
	GSdoeventsandreturn() ;
	if (fChanged) 
	    sw_displayallgraphs() ;
	return TRUE ;
}
	
bool cancel_change()
{
	GStogglecheck("col_menu", GSgetmenuitemname("col_menu", prev_node_color)) ;
	GStogglecheck("nd_type_menu", GSgetmenuitemname("nd_type_menu", prev_node_type)) ;
	GStogglecheck("nd_fill_menu", GSgetmenuitemname("nd_fill_menu", prev_node_fill)) ;
	GStogglecheck("ndth_menu", GSgetmenuitemname("ndth_menu", prev_node_lineth)) ;
	GStogglecheck("edth_menu", GSgetmenuitemname("edth_menu", prev_edge_lineth)) ;
	GStogglecheck("edlab_menu", GSgetmenuitemname("edlab_menu", prev_edge_label)) ;
	GStogglecheck("edcol_menu", GSgetmenuitemname("edcol_menu", prev_edge_color)) ;
	GStogglecheck("layout_menu", GSgetmenuitemname("layout_menu", prev_layout)) ;
	GStogglecheck("relayout_menu", GSgetmenuitemname("relayout_menu", prev_relayout)) ;
	GStogglecheck("laymode_menu", GSgetmenuitemname("layout_menu", prev_layoutmode)) ;
        GSenablemenuitem("layout_menu", "List") ;
        GSenablemenuitem("layout_menu", "CircleNet") ;
        GSenablemenuitem("layout_menu", "RootedTree") ;
        GSenablemenuitem("layout_menu", "KK_Net") ;
        GSenablemenuitem("layout_menu", "Hierarchy") ;

	GSremovenamedwindow("GraphAttr") ;
  	GSsetdeactivenamedwindow("GraphAttr");
	GSdoeventsandreturn() ;
	fGraphAttrActive = FALSE ;
	return TRUE ;
}

/*-------------------------------------------------------------------
  Following functions modify graphical attributes of a selected node
  -------------------------------------------------------------------*/
bool set_sn_type(char *item_name)
{
  int item_num = GSgetmenuitemnum("sn_type_menu", item_name);
	GStogglecheck("sn_type_menu", GSgetmenuitemname("sn_type_menu", prev_sn_type)) ;
	if (prev_sn_type != item_num) {
	prev_sn_type = item_num ;
	_nm_nodetype = _list_nodetype[item_num] ;
	}
	return TRUE ;
}

bool set_sn_fill(char *item_name)
{
  int item_num = GSgetmenuitemnum("sn_fill_menu", item_name);
	GStogglecheck("sn_fill_menu", GSgetmenuitemname("sn_fill_menu", prev_sn_fill)) ;
	if (prev_sn_fill != item_num) {
	prev_sn_fill = item_num ;
	_nm_nodefill = _list_nodefill[item_num] ;
	}
	return TRUE ;
}

bool set_sn_th(char *item_name)
{
  int item_num = GSgetmenuitemnum("sn_th_menu", item_name);
	GStogglecheck("sn_th_menu", GSgetmenuitemname("sn_th_menu", prev_sn_lineth)) ;
	if (prev_sn_lineth != item_num) {
	prev_sn_lineth = item_num ;
	_nm_nodelineth = _list_lineth[item_num] ;
	}
	return TRUE ;
}

bool set_sn_color(char *item_name)
{
  int item_num = GSgetmenuitemnum("sn_col_menu", item_name);
 	GStogglecheck("sn_col_menu", GSgetmenuitemname("sn_col_menu", prev_sn_color)) ;
	if (prev_sn_color != item_num) {
	prev_sn_color = item_num ;
	_nm_nodecolor = _list_color[item_num] ;
	}
	return TRUE ;
}

void show_nm_nodewidth()
{
	int	text_x, text_y, text_w, text_h ;
	GSColor	text_color ;
	char	temp_str[80] ;

	GSsetcurrentnamedwindow("NodeAttr") ;
	GSgetfieldrect("snwidth", &text_x, &text_y, &text_w, &text_h,
			&text_color) ;
	GSsetcolor(GSconvertcolor("white")) ;
	GSwfillrect(text_x, text_y, text_w, text_h) ;
	GSsetcolor(text_color) ;
	sprintf(temp_str, "%d", _nm_nodewidth) ;
	GSwwritemsg(text_x, text_y+10, temp_str) ;
	GSdoeventsandreturn() ;
}

void show_nm_nodeheight()
{
	int	text_x, text_y, text_w, text_h ;
	GSColor	text_color ;
	char	temp_str[80] ;

	GSsetcurrentnamedwindow("NodeAttr") ;
	GSgetfieldrect("snheight", &text_x, &text_y, &text_w, &text_h,
			&text_color) ;
	GSsetcolor(GSconvertcolor("white")) ;
	GSwfillrect(text_x, text_y, text_w, text_h) ;
	GSsetcolor(text_color) ;
	sprintf(temp_str, "%d", _nm_nodeheight) ;
	GSwwritemsg(text_x, text_y+10, temp_str) ;
	GSdoeventsandreturn() ;
}

bool sn_decwd()
{
	if (_nm_nodewidth > _SW_MINNODEWIDTH)
	    _nm_nodewidth-- ;
	show_nm_nodewidth() ;
	return TRUE ;
}
	
bool sn_incwd()
{
	if (_nm_nodewidth < _SW_MAXNODEWIDTH)
	    _nm_nodewidth++ ;
	show_nm_nodewidth() ;
	return TRUE ;
}
	
bool sn_decht()
{
	if (_nm_nodeheight > _SW_MINNODEHEIGHT)
	    _nm_nodeheight-- ;
	show_nm_nodeheight() ;
	return TRUE ;
}
	
bool sn_incht()
{
	if (_nm_nodeheight < _SW_MAXNODEHEIGHT)
	    _nm_nodeheight++ ;
	show_nm_nodeheight() ;
	return TRUE ;
}
	
bool sn_ok_change()
{
	bool		fChanged = FALSE ;

	// set modified variables back
	if (_nm_nodetype != (*_sw_pick_node).getType()) {
	    (*_sw_pick_node).setType(_nm_nodetype) ;
	    fChanged = TRUE ;
	    (*_sw_pick_graph).setNewLayoutFlag(TRUE) ;
	}
	if (_nm_nodecolor != (*_sw_pick_node).getColor()) {
	    (*_sw_pick_node).setColor(_nm_nodecolor) ;
	    fChanged = TRUE ;
	}
	if (_nm_nodefill != (*_sw_pick_node).getFilled()) {
	    (*_sw_pick_node).setFilled(_nm_nodefill) ; 
	    fChanged = TRUE ;
	}
	if (_nm_nodewidth != (*_sw_pick_node).getWidth()) {
	    (*_sw_pick_node).setWidth(_nm_nodewidth) ; 
	    fChanged = TRUE ;
	    (*_sw_pick_graph).setNewLayoutFlag(TRUE) ;
	}
	if (_nm_nodeheight != (*_sw_pick_node).getHeight()) {
	    (*_sw_pick_node).setHeight(_nm_nodeheight) ;     
	    fChanged = TRUE ;
	    (*_sw_pick_graph).setNewLayoutFlag(TRUE) ;
	}
	if (_nm_nodelineth != (*_sw_pick_node).getLineTH()) {
	    (*_sw_pick_node).setLineTH(_nm_nodelineth) ; 
	    fChanged = TRUE ;
	}

	GStogglecheck("sn_col_menu", GSgetmenuitemname("sn_col_menu", prev_sn_color)) ;
	GStogglecheck("sn_type_menu", GSgetmenuitemname("sn_type_menu", prev_sn_type)) ;
	GStogglecheck("sn_fill_menu", GSgetmenuitemname("sn_fill_menu", prev_sn_fill)) ;
	GStogglecheck("sn_th_menu", GSgetmenuitemname("sn_th_menu", prev_sn_lineth)) ;
        GSremovenamedwindow("NodeAttr") ;
  	GSsetdeactivenamedwindow("NodeAttr");
	fNodeAttrActive = FALSE ;
	GSsetcurrentnamedwindow(SWG_SHOWWIN) ;
	if (fChanged)
	    // (*_sw_pick_node).displayNode() ;
	    sw_displayallgraphs() ;
	return TRUE ;
}
	
bool sn_cancel_change()
{
	GStogglecheck("sn_col_menu", GSgetmenuitemname("sn_col_menu", prev_sn_color)) ;
	GStogglecheck("sn_type_menu", GSgetmenuitemname("sn_type_menu", prev_sn_type)) ;
	GStogglecheck("sn_fill_menu", GSgetmenuitemname("sn_fill_menu", prev_sn_fill)) ;
	GStogglecheck("sn_th_menu", GSgetmenuitemname("sn_th_menu", prev_sn_lineth)) ;
	GSremovenamedwindow("NodeAttr") ;
	GSsetdeactivenamedwindow("NodeAttr");
	fNodeAttrActive = FALSE ;
	return TRUE ;
}

/*--------------------------------------------------------------------
  Following functions modify graphical attributes of the selected edge
  --------------------------------------------------------------------*/
bool set_se_th(char *item_name)
{
	// static prev_se_lineth = -1 ;
  int item_num = GSgetmenuitemnum("se_th_menu", item_name);
	
	GStogglecheck("se_th_menu", GSgetmenuitemname("se_th_menu", prev_se_lineth)) ;
	if (prev_se_lineth != item_num) {
	prev_se_lineth = item_num ;
	_em_edgelineth = _list_lineth[item_num] ;
	}
	return TRUE ;
}

bool set_se_color(char *item_name)
{
  int item_num = GSgetmenuitemnum("se_col_menu", item_name);
	// static prev_se_color = -1 ;
	GStogglecheck("se_col_menu", GSgetmenuitemname("se_col_menu", prev_se_color)) ;
	if (prev_se_color != item_num) {
	prev_se_color = item_num ;
	_em_edgecolor = _list_color[item_num] ;
	}
	return TRUE ;
}

bool se_ok_change()
{
	bool		fChanged = FALSE ;

	// set modified variables back
	if (_em_edgecolor != (*_sw_pick_edge).getColor()) {
	    (*_sw_pick_edge).setColor(_em_edgecolor) ;
	    fChanged = TRUE ;
	}
	if (_em_edgelineth != (*_sw_pick_edge).getLineTH()) {
	    (*_sw_pick_edge).setLineTH(_em_edgelineth) ; 
	    fChanged = TRUE ;
	}
	GStogglecheck("se_col_menu", GSgetmenuitemname("se_col_menu", prev_se_color)) ;
	GStogglecheck("se_th_menu", GSgetmenuitemname("se_th_menu", prev_se_lineth)) ;
        GSremovenamedwindow("EdgeAttr") ;
  	GSsetdeactivenamedwindow("EdgeAttr");
	fEdgeAttrActive = FALSE ;
	GSsetcurrentnamedwindow(SWG_SHOWWIN) ;
	if (fChanged)
	    // (*_sw_pick_edge).displayEdge() ;
	    sw_displayallgraphs() ;
	return TRUE ;
}
	
bool se_cancel_change()
{
	GStogglecheck("se_col_menu", GSgetmenuitemname("se_col_menu", prev_se_color)) ;
	GStogglecheck("se_th_menu", GSgetmenuitemname("se_th_menu", prev_se_lineth)) ;
	GSremovenamedwindow("EdgeAttr") ;
  	GSsetdeactivenamedwindow("EdgeAttr");
	fEdgeAttrActive = FALSE ;
	return TRUE ;
}

/*---------------------------------------------------------------
  registration callback functions
  ---------------------------------------------------------------*/
extern "C" IntrFunction FuncNames[] = {
  {"set_format", (bool(*)())set_format}, 
  {"gfformat", (bool(*)())gfformat},
  {"canceldump", (bool(*)())canceldump},
  {"confirmdump", (bool(*)())confirmdump},
  {"dumpscreen", (bool(*)())dumpscreen},
  {"move_up", (bool(*)())move_up} ,
  {"move_down", (bool(*)())move_down} ,
  {"move_left", (bool(*)())move_left} ,
  {"move_right", (bool(*)())move_right} ,
  {"zoom_in", (bool(*)())zoom_in},
  {"zoom_out", (bool(*)())zoom_out},
  {"redraw", (bool(*)())redraw},
  {"do_swap", (bool(*)())do_swapnode},
  {"do_move", (bool(*)())do_movenode},
  {"do_insnode", (bool(*)())do_insnode},
  {"do_delnode", (bool(*)())do_delnode},
  {"do_insedge", (bool(*)())do_insedge},
  {"do_deledge", (bool(*)())do_deledge},
  {"do_savegraph", (bool(*)())do_savegraph},
  {"do_loadgraph", (bool(*)())do_loadgraph},
  {"doquit", (bool(*)())doquit},
  {"do_pick", (bool(*)())do_pick},
  {"do_remove", (bool(*)())do_remove},
  { "kbd_hndlr", (bool(*)())do_text },
  {"run_anim", (bool(*)())run_anim},
  {"step_anim", (bool(*)())step_anim},
  {"quit", (bool(*)())quit}, 
  {"set_node_type", (bool(*)())set_node_type} ,
  {"set_node_fill", (bool(*)())set_node_fill},
  {"set_node_th", (bool(*)())set_node_th}, 
  {"set_edge_th", (bool(*)())set_edge_th}, 
  {"set_edge_label", (bool(*)())set_edge_label}, 
  {"set_node_color", (bool(*)())set_node_color},
  {"set_edge_color", (bool(*)())set_edge_color},
  {"set_layout", (bool(*)())set_layout},
  {"set_relayout", (bool(*)())set_relayout},
  {"set_layoutmode", (bool(*)())set_layoutmode},
  {"dec_gx", (bool(*)())dec_gx},
  {"inc_gx", (bool(*)())inc_gx},
  {"dec_gy", (bool(*)())dec_gy},
  {"inc_gy", (bool(*)())inc_gy},
  {"dec_nodewd", (bool(*)())dec_nodewd},
  {"inc_nodewd", (bool(*)())inc_nodewd},
  {"dec_nodeht", (bool(*)())dec_nodeht},
  {"inc_nodeht", (bool(*)())inc_nodeht},
  {"dec_edgelen", (bool(*)())dec_edgelen},
  {"inc_edgelen", (bool(*)())inc_edgelen},
  {"ok_change", (bool(*)())ok_change},
  {"cancel_change", (bool(*)())cancel_change},
  {"set_sn_color", (bool(*)())set_sn_color},
  {"set_sn_type", (bool(*)())set_sn_type},
  {"set_sn_fill", (bool(*)())set_sn_fill},
  {"set_sn_th", (bool(*)())set_sn_th},
  {"sn_decwd", (bool(*)())sn_decwd},
  {"sn_incwd", (bool(*)())sn_incwd},
  {"sn_decht", (bool(*)())sn_decht},
  {"sn_incht", (bool(*)())sn_incht},
  {"sn_ok_change", (bool(*)())sn_ok_change},
  {"sn_cancel_change", (bool(*)())sn_cancel_change},
  {"set_se_th", (bool(*)())set_se_th},
  {"set_se_color", (bool(*)())set_se_color},
  {"se_ok_change", (bool(*)())se_ok_change},
  {"se_cancel_change", (bool(*)())se_cancel_change},
  {"set_gl_redraw", (bool(*)())set_gl_redraw},
  {"gl_ok_change", (bool(*)())gl_ok_change},
  {"gl_cancel_change", (bool(*)())gl_cancel_change},
  {"change_graph_attrs", (bool(*)())change_graph_attrs}, 
  {"change_node_attrs", (bool(*)())change_node_attrs}, 
  {"change_edge_attrs", (bool(*)())change_edge_attrs} ,
  {"change_global_attrs", (bool(*)())change_global_attrs} ,
  {"attr_menu", (bool(*)())do_nothing} ,
  {"edit_menu", (bool(*)())do_nothing} ,
  {"col_menu", (bool(*)())do_nothing} ,
  {"nd_type_menu", (bool(*)())do_nothing} ,
  {"nd_fill_menu", (bool(*)())do_nothing} ,
  {"ndth_menu", (bool(*)())do_nothing} ,
  {"edth_menu", (bool(*)())do_nothing} ,
  {"edlab_menu", (bool(*)())do_nothing} ,
  {"edcol_menu", (bool(*)())do_nothing} ,
  {"sn_col_menu", (bool(*)())do_nothing} ,
  {"sn_type_menu", (bool(*)())do_nothing} ,
  {"sn_fill_menu", (bool(*)())do_nothing} ,
  {"sn_th_menu", (bool(*)())do_nothing} ,
  {"se_th_menu", (bool(*)())do_nothing} ,
  {"se_col_menu", (bool(*)())do_nothing} ,
  {"file_menu", (bool(*)())do_nothing} ,
  {"layout_menu", (bool(*)())do_nothing} ,
  {"relayout_menu", (bool(*)())do_nothing} ,
  {"laymode_menu", (bool(*)())do_nothing} ,
  {"gl_redraw_menu", (bool(*)())do_nothing} ,
};

extern "C" int NumFuncs = sizeof(FuncNames)/sizeof(IntrFunction);

bool _sw_wininit(GS_MAIN_ARGS_F)
{
	if (!GSinitlog ("wt"))
  	{
	    /* printf("Unable to initialize output.log\n"); */
	    // delay(ERRORDELAY);
	    exit(-1);
    	}

/*  GSinterfaceinit(" ", "swan.inf", 
		   BSTORE, "w", G640x480x16, NULL, NULL, NULL, 0);*/
  GSinterfaceinit(" ", "swan.inf", 
		   BSTORE, "w", G640x480x16, GS_MAIN_ARGS_A);
  GSsetactivenamedwindow("root");
  GSsetactivenamedwindow(SWG_SHOWWIN) ;
  GSsetdeactivenamedwindow("GraphAttr");
  fGraphAttrActive = FALSE ;
  GSsetdeactivenamedwindow("NodeAttr") ;
  fNodeAttrActive = FALSE ;
  GSsetdeactivenamedwindow("EdgeAttr") ;
  fEdgeAttrActive = FALSE ;
  GSsetdeactivenamedwindow("GlobalAttr") ;
  fGlobalAttrActive = FALSE ;
  
  /*--- activate keyboard input function (do_text) ---*/
  GScharfunc("kbd_hndlr");

  GSmouseinit() ;
  GSdoeventsandreturn() ;

  return TRUE ;
}

void redraw_all()
{
  	int        text_x, text_y, text_wd, text_ht;
  	GSColor    text_color;

	swg_refresh() ;
	sw_print(OutputTextString) ;

	// redraw input string
        GSsetcurrentnamedwindow("root") ;
        GSgetfieldrect("MESSAGE", &text_x, &text_y, &text_wd, &text_ht,
                                                        &text_color) ;
        GSsetcolor(GSconvertcolor(color_table[DGRAY])) ;
        GSwfillrect(text_x, text_y+14, text_wd, text_ht-14) ;
        GSsetcolor(text_color) ;
        GSwwritemsg(text_x, text_y+24, InputTextString) ;

	if (fGraphAttrActive) {
	    GSredrawnamedwindow("GraphAttr") ;
	    redraw_graph_attr() ;
	}
	else if (fNodeAttrActive) {
	    GSredrawnamedwindow("NodeAttr") ;
	    redraw_node_attr() ;
	}
	else if (fEdgeAttrActive) {
	    GSredrawnamedwindow("EdgeAttr") ;
	    redraw_edge_attr() ;
	}
	else if (fGlobalAttrActive) {
	    GSredrawnamedwindow("GlobalAttr") ;
	    redraw_global_attr() ;
	}

    	switch (fProgState) {
    	case RUN:
	    showrunframe(TRUE) ;
	    break ;
    	case STEP:	
	    showstepframe(TRUE) ;
	    break ;
    	default:
	    break ;
    	}	
        GSdoeventsandreturn() ;
}

void redraw_coord(int x_pos, int y_pos)
{
	char	str[128] ;
	int	temp_x, temp_y ;
        int     text_x, text_y ;
        int     width, height ;
        GSColor color ;

        GSsetcurrentnamedwindow("root") ;
        GSgetfieldrect("COORDS", &text_x, &text_y, &width, &height,
                                                        &color) ;
        GSsetcolor(GSconvertcolor(color_table[LGRAY])) ;
        GSwfillrect(text_x, text_y, width, height) ;
        GSsetcolor(color) ;

	temp_x = (int)((x_pos+sc_x_origin) / zoom_factor) ;
	temp_y = (int)((y_pos+sc_y_origin) / zoom_factor) ;
	sprintf(str, "(%d, %d)", temp_x, temp_y) ;
        GSwwritemsg(text_x, text_y+10, str) ;
        GSsetactivenamedwindow(SWG_SHOWWIN) ;
	GSdoeventsandreturn() ;
}

void erase_coord(void)
{
        int     text_x, text_y ;
        int     width, height ;

        GSsetcurrentnamedwindow("root") ;
        GSgetfieldrect("COORDS", &text_x, &text_y, &width, &height,
                                                        NULL) ;
        GSsetcolor(GSconvertcolor(color_table[LGRAY])) ;
        GSwfillrect(text_x, text_y, width, height) ;
        GSsetactivenamedwindow(SWG_SHOWWIN) ;
	GSdoeventsandreturn() ;
}

/*
  gfformat - associated with popup menu which gives user a choice of
graphic
             file formats to dump to.
*/
bool gfformat (
  void
)
{
  // GSdisplayhelpmessage(18);
  return (TRUE);
}


//  Print screen to file

bool dumpscreen(void)
{
  BigString msgstring;

  GSremovenamedwindow("dump");
 // GSdisplayhelpmessage(Savemsg);
  DumpWinActive = FALSE;
/*
  if (!Use_bstore)
    redraw_all();
*/
  strcpy (DumpFileName, "swan");
  if (GraphFileFmt != NO_FILE)
    DumpSuccess = GSdumpscreen(DumpFileName, GraphFileFmt);
  else return FALSE;

  // GSdisplayhelpmessage(10);
/*
  DumpResultsActive = TRUE;
  GSdrawnamedwindow("alert", Use_bstore, POPUP);
  GSsetcolor(GSconvertcolor("black"));
  GSsetfont(LARGE);
*/
  if (DumpSuccess)
  {
    sprintf(msgstring, "The screen was saved to file: %s", DumpFileName);
    sw_print(msgstring) ;
    return TRUE;
  }
  else
  {
    sprintf(msgstring, "An error occurred saving the screen. No file was\
 saved.") ;
    sw_print(msgstring) ;
    GSbell();
    return FALSE;
  }
}

bool canceldump (
  void
)
{
  GSremovenamedwindow("dump");
  DumpWinActive = FALSE;
/*
  WarnedNotToCoverWin = FALSE;
  if (!Use_bstore) redraw_all();
  GSdisplayhelpmessage (Savemsg);
*/
  return TRUE;
}

void highlightfield (
  BigString  window,
  BigString  field,
  GSColor    bgcolor,
  GSColor    topleft,
  GSColor    bottomright
)
{
  int fx, fy, fwd, fht;

  GSgetfieldrect(field, &fx, &fy, &fwd, &fht, NULL);
  GSsetcurrentnamedwindow(window);
  GSsetcolor(bgcolor);
  GSwfillrect(fx, fy, fwd, fht);
  GSsetcolor(topleft);
  GSwdrawline(fx-1, fy-1, fx+fwd, fy-1);
  GSwdrawline(fx-2, fy-2, fx+fwd+1, fy-2);
  GSwdrawline(fx-1, fy-1, fx-1, fy+fht);
  GSwdrawline(fx-2, fy-2, fx-2, fy+fht+1);
  GSsetcolor(bottomright);
  GSwdrawline(fx, fy+fht, fx+fwd, fy+fht);
  GSwdrawline(fx-1, fy+fht+1, fx+fwd+1, fy+fht+1);
  GSwdrawline(fx+fwd, fy, fx+fwd, fy+fht);
  GSwdrawline(fx+fwd+1, fy-1, fx+fwd+1, fy+fht+1);
}

/*-------------------------------------------------------------------------------
  set_format: set a graphics file format for screen dump in response to user
              selection from GFformat popup menu
-------------------------------------------------------------------------------*/
bool set_format (
  int item_n
)
{
  int x, y;
  GSColor color;

  GSgetfieldrect("filetype", &x, &y, NULL, NULL, &color);
  highlightfield("dump", "filetype", GSconvertcolor("lgray"),
                 GSconvertcolor("black"), GSconvertcolor("white"));
  GSsetcolor(color);
  GSsetfont(LARGE);
  switch (item_n) {
    case 0:
      GraphFileFmt = BMP_16;
      GSwwritenamedmsg(x+5, y+SMALL+2, "bmp16msg");
      break;
    case 1:
      GraphFileFmt = PS_COLOR;
      GSwwritenamedmsg(x+5, y+SMALL+2, "pscolormsg");
      break;
    case 2:
      GraphFileFmt = PS_GRAY;
      GSwwritenamedmsg(x+5, y+SMALL+2, "psgraymsg");
      break;
    case 3:
      GraphFileFmt = GIF_16;
      GSwwritenamedmsg(x+5, y+SMALL+2, "gif16msg");
      break;
  }
  return (TRUE);
}

/*----------------------------------------------------------------------------
  Displays the "dump" window, warning the user that this will take a long
  time. The user is asked to confirm his or her wish to dump the screen to file
-----------------------------------------------------------------------------*/
bool confirmdump (
  void
)
{
  int x, y;
  GSColor color;

  // GSdisplayhelpmessage(17);
  DumpWinActive = TRUE;
  GSdrawnamedwindow("dump", BSTORE, POPUP);
  GSgetfieldrect("filetype", &x, &y, NULL, NULL, &color);
  highlightfield("dump", "filetype", GSconvertcolor("lgray"),
                 GSconvertcolor("black"), GSconvertcolor("white"));
  GSsetcolor(color);
  GSsetfont(LARGE);
  switch (GraphFileFmt) {
    case BMP_16:
      GSwwritenamedmsg(x+5, y+SMALL+2, "bmp16msg");
      break;
    case GIF_16:
      GSwwritenamedmsg(x+5, y+SMALL+2, "gif16msg");
      break;
    case PS_GRAY:
      GSwwritenamedmsg(x+5, y+SMALL+2, "psgraymsg");
      break;
    case PS_COLOR:
      GSwwritenamedmsg(x+5, y+SMALL+2, "pscolormsg");
      break;
    case NO_FILE: default:
      GSwwritenamedmsg(x+5, y+SMALL+2, "nofilemsg");
      break;
  }
  if (GraphFileFmt != NO_FILE)
  {
/*    GSgetfieldrect("dumpwarn1", &x, &y, NULL, NULL, &color);
    GSsetcolor(color);
    GSwwritenamedmsg(x, y, "dumpwarn1");
    GSgetfieldrect("dumpwarn2", &x, &y, NULL, NULL, &color);
    GSsetcolor(color);
    GSwwritenamedmsg(x, y, "dumpwarn2");
    GSgetfieldrect("dumpwarn3", &x, &y, NULL, NULL, &color);
    GSsetcolor(color);
    GSwwritenamedmsg(x, y, "dumpwarn3");*/
  }
  return TRUE;
}

/************************************************************
 * End of event.c				            *
 ************************************************************/
