/**********************************************************
 * listlay.c --- a layout algorithm for linked lists      *
 * Jun Yang, November 17, 1994				  *
 **********************************************************/

/*--------------- Header Files ---------------*/

#include <stdio.h>
#include <math.h>
#include "swan.h"
#include "laylist.h"

#define NUMVERTEX	128	

//-------------------
//  List_layout -- the main function for List layout algorithm
//___________________

bool List_layout(BasicGraph* g_ptr)
{
	BasicLayoutComp		*temp_lc ;
	NODELIST		*temp_nodelist ;
	BasicNode		*temp_node ;
	bool			ok ;
	

	if (g_ptr == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    return FALSE ;
	}

	if ((*(*g_ptr).getNodeList()).getNodeCount() > NUMVERTEX)
	    return FALSE ;

	if ((*(*g_ptr).getNodeList()).getNodeCount() == 0)
	    return TRUE ;

	temp_nodelist = (*g_ptr).getNodeList() ;
	ok = (*temp_nodelist).getFirstNode(temp_node) ;
	while (ok) {
	    (*temp_node).setRPos(UnsetPos) ;
	    ok = (*temp_nodelist).getNextNode(temp_node) ;
	}
	
	temp_lc = (*g_ptr).getRootLC() ;
	_cal_posandbox(g_ptr, temp_lc) ;
	return TRUE ; 

} /* end of List_layout */

bool _cal_posandbox(BasicGraph* g_ptr, BasicLayoutComp* lc_ptr) 
{
	BasicLayoutComp		*cur_lc ;
	bool			ok ;
	LCNBLIST		*temp_lclist ;
	NODELIST		*temp_nodelist ;
	BasicNode		*cur_node, *next_node, *temp_node ;
	LAYOUTCOMPTYPE		cur_lc_type ;
	NODENBLIST		*temp_nblist ;
	struct _nb_node		*temp_nb ;
	LOCATION		cur_pos, pos2 ;
	BasicLayoutComp		*nb_lc, *temp_lc ;
	BOXSIZE			temp_size, temp_size2 ;
	BOXSIZE			cur_nodesize ;
	int			cur_edgelength ;

	if (lc_ptr == NULL) 
	    return FALSE ;
	cur_lc_type = (*lc_ptr).getType() ;
	if ((cur_lc_type != LISTACROSS) && (cur_lc_type != LISTDOWN)
	    && (cur_lc_type != ARRAYACROSS)  
	    && (cur_lc_type != ARRAYDOWN))
	    return FALSE ;

	temp_lclist = (*lc_ptr).getChLCList() ;
	ok = (*temp_lclist).getFirstNode(cur_lc) ;
	while (ok) {
	    (*cur_lc).setRPos(UnsetPos) ;
	    if (!_cal_posandbox(g_ptr, cur_lc))
		return FALSE ;
	    ok = (*temp_lclist).getNextNode(cur_lc) ;
	}

	cur_pos.x = cur_pos.y = 0 ;
	temp_size.w = temp_size.h = 0 ;
	cur_edgelength = (*g_ptr).getEdgeLength() ;
	next_node = NULL ;
	nb_lc = NULL ;
	cur_node = (*lc_ptr).getFNode() ;
	    // start from the fnode
	while (cur_node != NULL) {
	    (*cur_node).setRPos(cur_pos) ;
	    cur_nodesize.w = (*cur_node).getWidth() ;
	    cur_nodesize.h = (*cur_node).getHeight() ;
	    switch (cur_lc_type) {
	    case LISTDOWN:
	    case ARRAYDOWN:
		if (cur_nodesize.w > temp_size.w)
		    temp_size.w = cur_nodesize.w ;
		break ;
	    case LISTACROSS:
	    case ARRAYACROSS:
		if (cur_nodesize.h > temp_size.h)
		    temp_size.h = cur_nodesize.h ;
		break ;
	    }
	    temp_nblist = (*cur_node).getNBList() ;
	    ok = (*temp_nblist).getFirstNode(temp_nb) ;
	    while (ok) {
		temp_node = temp_nb->id ;
		temp_lc = (*temp_node).getLayoutComp() ;
		if (temp_lc == lc_ptr) {
		    if (next_node == NULL && 
			(*temp_node).getRPos() == UnsetPos) {
			next_node = temp_node ;
		    }
		}	
		else if (nb_lc == NULL) { // two nodes in different LC 
		    if (cur_node == (*temp_lc).getPNode()) {
			switch (cur_lc_type) {
			case LISTDOWN:
			case ARRAYDOWN:
			    pos2.x = cur_pos.x + cur_nodesize.w
			               + cur_edgelength ;
			    pos2.y = cur_pos.y ;
			    (*temp_lc).setRPos(pos2) ;
			    break ;
			case LISTACROSS:
			case ARRAYACROSS:
			    pos2.x = cur_pos.x ;
			    pos2.y = cur_pos.y + cur_nodesize.h
				       + cur_edgelength ;
			    (*temp_lc).setRPos(pos2) ;
			    break ;
			}
			nb_lc = temp_lc ;
		    }
		}	
	        ok = (*temp_nblist).getNextNode(temp_nb) ;
	    }

	    temp_size2.w = temp_size2.h = 0 ;
	    switch (cur_lc_type) {
	    case LISTDOWN:
		// modify box size in X-direction
	        if (nb_lc != NULL) {
		    temp_size2 = (*nb_lc).getBoxSize() ;
		    if (temp_size2.w + cur_nodesize.w + cur_edgelength >
			temp_size.w)
			temp_size.w = temp_size2.w + cur_nodesize.w +
					cur_edgelength ;
		}

		// move in Y-direction
		if (temp_size2.h > cur_nodesize.h + cur_edgelength)
		    cur_pos.y += temp_size2.h ;
		else 
		    cur_pos.y += cur_nodesize.h + cur_edgelength ;
		break ;

	    case ARRAYDOWN:
		// modify box size in X-direction
	        if (nb_lc != NULL) {
		    temp_size2 = (*nb_lc).getBoxSize() ;
		    if (temp_size2.w + cur_nodesize.w + cur_edgelength>
			temp_size.w)
			temp_size.w = temp_size2.w + cur_nodesize.w +
					cur_edgelength ;
		}

		// move in Y-direction
		cur_pos.y += cur_nodesize.h - 1 ;
		break ;

	    case LISTACROSS:
		// modify box size in Y-direction
	        if (nb_lc != NULL) {
		    temp_size2 = (*nb_lc).getBoxSize() ;
		    if (temp_size2.h + cur_nodesize.h + cur_edgelength >
			temp_size.h)
			temp_size.h = temp_size2.h + cur_nodesize.h +
					cur_edgelength ;
		}

		// move in X-direction
		if (temp_size2.w > cur_nodesize.w + cur_edgelength)
		    cur_pos.x += temp_size2.w ;
		else 
		    cur_pos.x += cur_nodesize.w + cur_edgelength ;
		break ;

	    case ARRAYACROSS:
		// modify box size in Y-direction
	        if (nb_lc != NULL) {
		    temp_size2 = (*nb_lc).getBoxSize() ;
		    if (temp_size2.h + cur_nodesize.h + cur_edgelength >
			temp_size.h)
			temp_size.h = temp_size2.h + cur_nodesize.h +
					cur_edgelength ;
		}

		// move in X-direction
		cur_pos.x += cur_nodesize.w - 1;
		break ;

	    default:
		return FALSE ;
	    }
		
	    cur_node = next_node ;
	    next_node = NULL ;
	    nb_lc = NULL ;
	}

	// process unassigned components
	next_node = NULL ;
	nb_lc = NULL ;
	temp_nodelist = (*lc_ptr).getChNodeList() ;
	ok = (*temp_nodelist).getFirstNode(cur_node) ;
	while (ok) {
	  if ((*cur_node).getRPos() == UnsetPos) {
	    (*cur_node).setRPos(cur_pos) ;
	    cur_nodesize.w = (*cur_node).getWidth() ;
	    cur_nodesize.h = (*cur_node).getHeight() ;
	    switch (cur_lc_type) {
	    case LISTDOWN:
	    case ARRAYDOWN:
		if (cur_nodesize.w > temp_size.w)
		    temp_size.w = cur_nodesize.w ;
		break ;
	    case LISTACROSS:
	    case ARRAYACROSS:
		if (cur_nodesize.h > temp_size.h)
		    temp_size.h = cur_nodesize.h ;
		break ;
	    }
	    temp_nblist = (*cur_node).getNBList() ;
	    ok = (*temp_nblist).getFirstNode(temp_nb) ;
	    while (ok) {
		temp_node = temp_nb->id ;
		temp_lc = (*temp_node).getLayoutComp() ;
		if (temp_lc == lc_ptr) {
		    if (next_node == NULL && 
			(*temp_node).getRPos() == UnsetPos) {
			next_node = temp_node ;
		    }
		}	
		else if (nb_lc == NULL) { // two nodes in different LC 
		    if (cur_node == (*temp_lc).getPNode()) {
			switch (cur_lc_type) {
			case LISTDOWN:
			case ARRAYDOWN:
			    pos2.x = cur_pos.x + cur_nodesize.w
			               + cur_edgelength ;
			    pos2.y = cur_pos.y ;
			    (*temp_lc).setRPos(pos2) ;
			    break ;
			case LISTACROSS:
			case ARRAYACROSS:
			    pos2.x = cur_pos.x ;
			    pos2.y = cur_pos.y + cur_nodesize.h
				       + cur_edgelength ;
			    (*temp_lc).setRPos(pos2) ;
			    break ;
			}
			nb_lc = temp_lc ;
		    }
		}	
	        ok = (*temp_nblist).getNextNode(temp_nb) ;
	    }

	    temp_size2.w = temp_size2.h = 0 ;
	    switch (cur_lc_type) {
	    case LISTDOWN:
		// modify box size in X-direction
	        if (nb_lc != NULL) {
		    temp_size2 = (*nb_lc).getBoxSize() ;
		    if (temp_size2.w + cur_nodesize.w + cur_edgelength >
			temp_size.w)
			temp_size.w = temp_size2.w + cur_nodesize.w +
					cur_edgelength ;
		}

		// move in Y-direction
		if (temp_size2.h > cur_nodesize.h + cur_edgelength)
		    cur_pos.y += temp_size2.h ;
		else 
		    cur_pos.y += cur_nodesize.h + cur_edgelength ;
		break ;

	    case ARRAYDOWN:
		// modify box size in X-direction
	        if (nb_lc != NULL) {
		    temp_size2 = (*nb_lc).getBoxSize() ;
		    if (temp_size2.w + cur_nodesize.w + cur_edgelength >
			temp_size.w)
			temp_size.w = temp_size2.w + cur_nodesize.w +
					cur_edgelength ;
		}

		// move in Y-direction
		cur_pos.y += cur_nodesize.h ;
		break ;

	    case LISTACROSS:
		// modify box size in Y-direction
	        if (nb_lc != NULL) {
		    temp_size2 = (*nb_lc).getBoxSize() ;
		    if (temp_size2.h + cur_nodesize.h + cur_edgelength >
			temp_size.h)
			temp_size.h = temp_size2.h + cur_nodesize.h +
					cur_edgelength ;
		}

		// move in X-direction
		if (temp_size2.w > cur_nodesize.w + cur_edgelength)
		    cur_pos.x += temp_size2.w ;
		else 
		    cur_pos.x += cur_nodesize.w + cur_edgelength ;
		break ;

	    case ARRAYACROSS:
		// modify box size in Y-direction
	        if (nb_lc != NULL) {
		    temp_size2 = (*nb_lc).getBoxSize() ;
		    if (temp_size2.h + cur_nodesize.h + cur_edgelength >
			temp_size.h)
			temp_size.h = temp_size2.h + cur_nodesize.h +
					cur_edgelength ;
		}

		// move in X-direction
		cur_pos.x += cur_nodesize.w + cur_edgelength ;
		break ;

	    default:
		return FALSE ;
	    }
		
	    if (next_node == NULL)
		break ;
	    cur_node = next_node ;
	    next_node = NULL ;
	    nb_lc = NULL ;
	  }
	  else 
	      ok = (*temp_nodelist).getNextNode(cur_node) ;
	}

	// cal box size
	ok = (*temp_lclist).getFirstNode(cur_lc) ;
	while (ok) {
	    if ((*cur_lc).getRPos() == UnsetPos) {
		switch (cur_lc_type) {
		case LISTDOWN:
		case ARRAYDOWN:
		    pos2.x = cur_pos.x ;
		    pos2.y = cur_pos.y + cur_edgelength ;
		    (*cur_lc).setRPos(pos2) ;
		    temp_size2 = (*cur_lc).getBoxSize() ;
		    if (temp_size2.w > temp_size.w)
			temp_size.w = temp_size2.w ;
		    cur_pos.y += cur_edgelength + temp_size2.h ;
		    break ;
		case LISTACROSS:
		case ARRAYACROSS:
		    pos2.x = cur_pos.x + cur_edgelength ;
		    pos2.y = cur_pos.y ;
		    (*cur_lc).setRPos(pos2) ;
		    temp_size2 = (*cur_lc).getBoxSize() ;
		    if (temp_size2.h > temp_size.h)
			temp_size.h = temp_size2.h ;
		    cur_pos.x += cur_edgelength + temp_size2.w ;
		    break ;
		default:
		    break ;
		}
	    }
	    ok = (*temp_lclist).getNextNode(cur_lc) ;
	}

	switch (cur_lc_type) {
	case LISTDOWN:
	case ARRAYDOWN:
	    temp_size.h = cur_pos.y ;
	    break ;
	case LISTACROSS:
	case ARRAYACROSS:
	    temp_size.w = cur_pos.x ;
	    break ;
	default:
	    break ;
	}
	(*lc_ptr).setBoxSize(temp_size) ;
	return TRUE ;
} 
 
bool _List_edge_ppath(BasicGraph* g_ptr)
{
	BasicEdge*	temp_edge ;
	BasicNode	*s_node, *e_node ;
	EDGELIST*	temp_edgelist ;
	bool		ok ;
	CURVE		*ppath ;
	BasicLayoutComp *lc1, *lc2, *lc_id ;
        LOCATION        spos, epos, pos1, pos2, pos3, pos4, lc_pos ;
	BOXSIZE		ssize, esize ;
        Stack<SWPOINT>    temp_sk ;
        LAYOUTCOMPID    temp_lc ;
        int             con_bus, bus1, t1, t2 ;

	temp_edgelist = (*g_ptr).getEdgeList() ;
	ok = (*temp_edgelist).getFirstNode(temp_edge) ;
	while (ok) {
	    lc_id = (*temp_edge).getLayoutComp() ;
	    s_node = (*temp_edge).getSNode() ;
	    e_node = (*temp_edge).getENode() ;
	    spos = (*s_node).getPPos() ;
	    epos = (*e_node).getPPos() ;
	    lc1 = (*s_node).getLayoutComp() ;
	    lc2 = (*e_node).getLayoutComp() ;
	  
	    ssize = ((*s_node).getAttr())->size ;
	    esize = ((*e_node).getAttr())->size ;
	    ppath = (*temp_edge).getPPath() ;
	    (*ppath).clear() ;

	  if (spos != UnsetPos && epos != UnsetPos) {
	  if (lc1 == lc2) {
	    switch ((*lc1).getType()) {
	    case LISTACROSS:
		switch ((*lc1).getNodeType()) {
		case BOX:
		    pos1.x = spos.x + ssize.w ;
		    pos1.y = spos.y + ssize.h/2 ;
		    (*ppath).insert(pos1) ;
                    pos2.x = epos.x ;
                    pos2.y = pos1.y ;
                    (*ppath).insert(pos2) ;
		    break ;
		case TABOXL:
		    pos1.x = spos.x + ssize.w*7/8 ;
		    pos1.y = spos.y + ssize.h/2 ;
		    (*ppath).insert(pos1) ;
                    pos2.x = epos.x ;
                    pos2.y = pos1.y ;
                    (*ppath).insert(pos2) ;
		    break ;
		default:
		    break ;
		}    
		break ;

	    case LISTDOWN:
		switch ((*lc1).getNodeType()) {
		case BOX:
		    pos1.x = spos.x + ssize.w/2 ;
		    pos1.y = spos.y + ssize.h ;
		    (*ppath).insert(pos1) ;
                    pos2.x = pos1.x ;
                    pos2.y = epos.y ;
                    (*ppath).insert(pos2) ;
		    break ;
		case TABOXL:
		    pos1.x = spos.x + ssize.w*5/8 ;
		    pos1.y = spos.y + ssize.h/2 ;
		    (*ppath).insert(pos1) ;
                    pos2.x = pos1.x ;
                    pos2.y = epos.y ;
                    (*ppath).insert(pos2) ;
		    break ;
		default:
		    break ;
		}
		break ;
	    
	    case ARRAYDOWN:
	    case ARRAYACROSS:
		break ;

	    default:
		return FALSE ;
	    }
	  }
	  else { // two nodes are in different LC
	  if (e_node == (*lc2).getFNode() &&
	      (*lc2).getPNode() == s_node) { // e_node is the flag node
	    switch ((*lc1).getType()) {

	    case LISTDOWN:
	    case ARRAYDOWN:
		switch ((*lc1).getNodeType()) {
		case BOX:
		    pos1.x = spos.x + ssize.w ;
		    pos1.y = spos.y + ssize.h/2 ;
		    (*ppath).insert(pos1) ;
                    pos2.x = epos.x ;
                    pos2.y = pos1.y ;
                    (*ppath).insert(pos2) ;
		    break ;
		case TABOXL:
		    pos1.x = spos.x + ssize.w*7/8 ;
		    pos1.y = spos.y + ssize.h/2 ;
		    (*ppath).insert(pos1) ;
                    pos2.x = epos.x ;
                    pos2.y = pos1.y ;
                    (*ppath).insert(pos2) ;
		    break ;
		default:
		    break ;
		}
		break ;

	    case LISTACROSS:
	    case ARRAYACROSS:
		switch ((*lc1).getNodeType()) {
		case BOX:
		    pos1.x = spos.x + ssize.w/2 ;
		    pos1.y = spos.y + ssize.h ;
		    (*ppath).insert(pos1) ;
                    pos2.x = pos1.x ;
                    pos2.y = epos.y ;
                    (*ppath).insert(pos2) ;
		    break ;
		case TABOXL:
		    pos1.x = spos.x + ssize.w*5/8 ;
		    pos1.y = spos.y + ssize.h/2 ;
		    (*ppath).insert(pos1) ;
                    pos2.x = pos1.x ;
                    pos2.y = epos.y ;
                    (*ppath).insert(pos2) ;
		    break ;
		default:
		    break ;
		}
	 	break ;

	    default:
		return FALSE ;
	    }
	  }
	  else {  // connection between nodes in different LC
	    if (lc_id != NULL) { // CCA already set
		if ((*lc1).getType() == LISTDOWN ||
		   (*lc1).getType() == LISTACROSS) {
		switch ((*lc1).getType()) {
		case LISTDOWN:
		    if ((bus1 = (*s_node).getBBus()) == -1) 
		        return FALSE ;
		    pos1.x = spos.x + ssize.w - BUSDIST * bus1 ;
		    pos1.y = spos.y + ssize.h ;
		    if ((*s_node).getType() == CIRCLE) {
			t1 = ssize.w/2 ;
			t2 = t1 - BUSDIST * bus1 ;
		        pos1.y = (int)(sqrt(t1 * t1 - t2 * t2)
				* ssize.h / ssize.w) + spos.y +
				ssize.h/2 ; 
		    }
		    break ;
			
		case LISTACROSS:
		    if ((bus1 = (*s_node).getRBus()) == -1) 
		        return FALSE ;
		    pos1.x = spos.x + ssize.w ;
		    pos1.y = spos.y + ssize.h - BUSDIST * bus1 ;
		    if ((*s_node).getType() == CIRCLE) {
			t1 = ssize.h/2 ;
			t2 = t1 - BUSDIST * bus1 ;
		        pos1.x = (int)(sqrt(t1 * t1 - t2 * t2)
				* ssize.w / ssize.h) + spos.x +
				ssize.w/2 ; 
		    }
		    break ;

		default:
		    return FALSE ; // wrong type
		}
		(*ppath).insert(pos1) ;
		if ((temp_lc = (*lc1).searchNBLC(s_node)) != NULL) {
		    switch ((*lc1).getType()) {
		    case LISTDOWN:
			pos1.y = spos.y + ((*temp_lc).getBoxSize()).h +
				 BUSDIST * bus1 ;
		 	break ;
		    case LISTACROSS:
			pos1.x = spos.x + ((*temp_lc).getBoxSize()).w +
				 BUSDIST * bus1 ;
			break ;
		    }
		}
		else {
		    switch ((*lc1).getType()) {
		    case LISTDOWN:
			pos1.y += BUSDIST * bus1 ;
		 	break ;
		    case LISTACROSS:
			pos1.x += BUSDIST * bus1 ;
			break ;
		    }
		}
		(*ppath).insert(pos1) ;	
		temp_lc = lc1 ;
		do {
		    switch ((*temp_lc).getType()) {
		    case LISTDOWN:
		        if ((bus1 = (*temp_lc).getRBus()) == -1)
			    return FALSE ;
			pos2.x = ((*temp_lc).getPPos()).x + 
				 ((*temp_lc).getBoxSize()).w -
				 BUSDIST * bus1 ;
			pos2.y = pos1.y ;
			break ;
		    case LISTACROSS:
		        if ((bus1 = (*temp_lc).getBBus()) == -1)
			    return FALSE ;
			pos2.x = pos1.x ;
			pos2.y = ((*temp_lc).getPPos()).y + 
				 ((*temp_lc).getBoxSize()).h -
				 BUSDIST * bus1 ;
			break ;
		    default:
			return FALSE ;
		    }
		    (*ppath).insert(pos2) ;
		    pos1 = pos2 ;
		    temp_lc = (*temp_lc).getPID() ;
		}
		while (temp_lc != lc_id) ;
		switch ((*temp_lc).getType()) {
		case LISTDOWN:
		    if ((bus1 = (*temp_lc).getRBus()) == -1)
		        return FALSE ;
		    pos2.x = ((*temp_lc).getPPos()).x + 
			     ((*temp_lc).getBoxSize()).w - 
			     BUSDIST * bus1 ;
		    pos2.y = pos1.y ;
		    break ;
		case LISTACROSS:
		    if ((bus1 = (*temp_lc).getBBus()) == -1)
		        return FALSE ;
		    pos2.x = pos1.x ;
		    pos2.y = ((*temp_lc).getPPos()).y + 
			     ((*temp_lc).getBoxSize()).h - 
			     BUSDIST * bus1 ;
		    break ;
		default:
		    return FALSE ;
		}
		(*ppath).insert(pos2) ;
		con_bus = bus1 ;

		// path from the end node 
		switch ((*lc2).getType()) {
		case LISTDOWN:
		    if ((bus1 = (*e_node).getBBus()) == -1) 
		        return FALSE ;
		    pos1.x = epos.x + esize.w - BUSDIST * bus1 ;
		    pos1.y = epos.y + esize.h ;
		    if ((*e_node).getType() == CIRCLE) {
			t1 = esize.w/2 ;
			t2 = t1 - BUSDIST * bus1 ;
		        pos1.y = (int)(sqrt(t1 * t1 - t2 * t2)
				* esize.h / esize.w) + 
				epos.y + esize.h/2 ; 
		    }
		    break ;

		case LISTACROSS:
		    if ((bus1 = (*e_node).getRBus()) == -1) 
		        return FALSE ;
		    pos1.x = epos.x + esize.w ;
		    pos1.y = epos.y + esize.h - BUSDIST * bus1 ;
		    if ((*e_node).getType() == CIRCLE) {
			t1 = esize.h/2 ;
			t2 = t1 - BUSDIST * bus1 ;
		        pos1.x = (int)(sqrt(t1 * t1 - t2 * t2)
				* esize.w / esize.h) + 
				epos.x + esize.w/2 ; 
		    }
		    break ;

		default:
		    return FALSE ; // wrong type
		}
		temp_sk.push(pos1) ;
		if ((temp_lc = (*lc2).searchNBLC(e_node)) != NULL) {
		    switch ((*lc2).getType()) {
		    case LISTDOWN:
			pos1.y = epos.y + ((*temp_lc).getBoxSize()).h -
				 BUSDIST * bus1 ;
		 	break ;
		    case LISTACROSS:
			pos1.x = epos.x + ((*temp_lc).getBoxSize()).w -
				 BUSDIST * bus1 ;
		    }
		}
		else {
		    switch ((*lc2).getType()) {
		    case LISTDOWN:
			pos1.y += BUSDIST * bus1 ;
		 	break ;
		    case LISTACROSS:
			pos1.x += BUSDIST * bus1 ;
			break ;
		    default:
			return FALSE ;
		    }
		}
		temp_sk.push(pos1) ;
		temp_lc = lc2 ;
		do {
		    switch ((*temp_lc).getType()) {
		    case LISTDOWN:
		        if ((bus1 = (*temp_lc).getRBus()) == -1)
			    return FALSE ;
			pos2.x = ((*temp_lc).getPPos()).x + 
				 ((*temp_lc).getBoxSize()).w -
				 BUSDIST * bus1 ; 
			pos2.y = pos1.y ;
			break ;
		    case LISTACROSS:
		        if ((bus1 = (*temp_lc).getBBus()) == -1)
			    return FALSE ;
			pos2.x = pos1.x ;
			pos2.y = ((*temp_lc).getPPos()).y + 
				 ((*temp_lc).getBoxSize()).h - 
				 BUSDIST * bus1 ;
			break ;
		    default:
			return FALSE ;
		    }
		    temp_sk.push(pos2) ;
		    pos1 = pos2 ;
		    temp_lc = (*temp_lc).getPID() ;
		}
		while (temp_lc != lc_id) ;

		// use con_bus 
		switch ((*temp_lc).getType()) {
		case LISTDOWN:
		    pos2.x = ((*temp_lc).getPPos()).x + 
			     ((*temp_lc).getBoxSize()).w -
			     BUSDIST * con_bus ;
		    pos2.y = pos1.y ;
		    break ;
		case LISTACROSS:
		    pos2.x = pos1.x ;
		    pos2.y = ((*temp_lc).getPPos()).y + 
			     ((*temp_lc).getBoxSize()).h -
			     BUSDIST * con_bus ;
		    break ;
		default:
		    return FALSE ;
		}
		temp_sk.push(pos2) ;
	
		// connect the two segments	
		while (temp_sk.pop(pos1)) 
		    (*ppath).insert(pos1) ;	
	    }
	    }
	    else { // arbitrary connection
		pos1.x = spos.x ;
		pos1.y = spos.y ;
		(*ppath).insert(pos1) ;
                pos2.x = epos.x ;
                pos2.y = epos.y ;
		(*ppath).insert(pos2) ;
	   }
	}
	}
	}
	  ok = (*temp_edgelist).getNextNode(temp_edge) ;
	}
	return TRUE ;	
}
	    
/**********************************************************
 * End of listlay.c     			          *
 **********************************************************/
