/***********************************************************
 * manual.c -- a layout algorithm for a general graphs     *
 * Jun Yang, March 9, 1995     		 		   *
 ***********************************************************/

/*--------------- Header Files ---------------*/

#include <stdio.h>
#include <math.h>
#include "swan.h"
#include "layman.h"
#include "laycn.h"

#define NUMVERTEX	128	

//-------------------
//  Manual_layout -- the main function for manual layout algorithm
//___________________

bool Manual_layout(BasicGraph* g_ptr)
{
	BasicLayoutComp		*temp_lc ;
	BasicLayoutComp		*root_lc ;
	LCNBLIST		*temp_lclist ;

	if (g_ptr == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    return FALSE ;
	}

	if ((*(*g_ptr).getNodeList()).getNodeCount() > NUMVERTEX)
	    return FALSE ;

	if ((*(*g_ptr).getNodeList()).getNodeCount() == 0)
	    return TRUE ;

	root_lc = (*g_ptr).getRootLC() ;
	temp_lclist = (*root_lc).getChLCList() ;
	if (!(*temp_lclist).getFirstNode(temp_lc))
	    return FALSE ;

	if ((*temp_lc).getType() != MANUAL)
	    return FALSE ;

	if (!_cal_Manual_box(g_ptr, temp_lc))
	    return FALSE ;
	(*root_lc).setBoxSize((*temp_lc).getBoxSize()) ;

	// there is no need to calculate positions of nodes in 
	// MANUAL LC
	return TRUE ; 

} /* end of Manual_layout */

bool _cal_Manual_box(BasicGraph* g_ptr, BasicLayoutComp* lc_ptr) 
{
	NODELIST	*temp_nodelist ;
	BasicNode	*temp_node ;
	BOXSIZE		temp_size ;
	LOCATION	temp_pos ;
	int		max_x=0, max_y=0 ;
	bool		ok ;

	temp_nodelist = (*lc_ptr).getChNodeList() ;
	ok = (*temp_nodelist).getFirstNode(temp_node) ;
	while (ok) {
	    temp_size = (*temp_node).getBoxSize() ;
	    temp_pos = (*temp_node).getRPos() ;
	    if (temp_pos.x + temp_size.w > max_x)
		max_x = temp_pos.x + temp_size.w ;
	    if (temp_pos.y + temp_size.h > max_y)
		max_y = temp_pos.y + temp_size.h ;
	    ok = (*temp_nodelist).getNextNode(temp_node) ;
	}
	temp_size.w = max_x ;	
	temp_size.h = max_y ;
	(*lc_ptr).setBoxSize(temp_size) ;
	return TRUE ;
}

inline int abs(int x) { if (x < 0) return x * -1; else return x; }

bool _Manual_edge_ppath(BasicGraph* g_ptr)
{
        bool         ok ;
        BasicEdge*      temp_edge ;
        BasicNode       *s_node, *e_node ;
        EDGELIST*       temp_edgelist ;
        CURVE           *ppath ;
        LOCATION        spos, epos, pos1, pos2 ;
	BasicLayoutComp *lc1, *lc2 ;
  
        temp_edgelist = (*g_ptr).getEdgeList() ;
        ok = (*temp_edgelist).getFirstNode(temp_edge) ;
        while (ok) {
            s_node = (*temp_edge).getSNode() ;
            e_node = (*temp_edge).getENode() ;
            lc1 = (*s_node).getLayoutComp() ;
            lc2 = (*e_node).getLayoutComp() ;
            ppath = (*temp_edge).getPPath() ;
            (*ppath).clear() ;
	    spos = (*s_node).getPPos() ;
	    epos = (*e_node).getPPos() ;
	    if (spos != UnsetPos && epos!= UnsetPos) {
	    if (lc1 == lc2) {
		if ((*temp_edge)._intersections(spos, 
		    (*s_node).getType(), (*s_node).getBoxSize(),
		    epos, (*e_node).getType(),
		    (*e_node).getBoxSize(), pos1, pos2)) {

		   // Following section added by Jeff Nielsen
		   // to adjust edges that go in both directions

		   if (g_ptr->searchEdge(e_node->getID(), s_node->getID())
		       != NULL) {
		      // Means there's an edge in the other direction
		      float slope;
		      if (pos1.x == pos2.x) slope = 100.0;
		      else slope = float(abs(pos1.y - pos2.y)) /
			 (abs(pos1.x - pos2.x));
		      
		      // if (pos1.y == pos2.y)
		      if (slope < 0.5) {  // Horizontal or diagonal
			 if (pos1.x < pos2.x)
			    { pos1.y -= 2; pos2.y -= 2; }
			 else
			    { pos1.y += 2; pos2.y += 2; }
		      }
		      else {		  // Vertical or diagonal line
			 if (pos1.y > pos2.y)
			    { pos1.x -= 3; pos2.x -= 3; }
			 else
			    { pos1.x += 3; pos2.x += 3; }
		      }
		   }
		   // End section added by Jeff Nielsen

		   (*ppath).insert(pos1);
		   (*ppath).insert(pos2);
		}
	    }
	    else 
		return FALSE ;
	    }
            ok = (*temp_edgelist).getNextNode(temp_edge) ;
	}
	_Manual_edgelabel_pos(g_ptr) ;
	return TRUE ;
}

bool _Manual_edgelabel_pos(BasicGraph* g_ptr) 
{
        bool         ok, ok2 ;
        BasicEdge       *temp_edge1, *temp_edge2, *temp_edge3 ;
        BasicNode       *s_node1, *e_node1, *s_node2, *e_node2 ;
        EDGELIST*       temp_edgelist ;
        LOCATION        spos, epos, spos2, epos2 ;
	LOCATION	pos1, pos2, pos3, pos4 ;
        EDGEINFO        *temp_info ;
	double		cur_t ;
	LABEL		temp_label ;
	BOXSIZE		temp_size ;
	SortedDoubleList<double>  *temp_ipt_list ;

	temp_ipt_list = new SortedDoubleList<double> ;
	temp_edgelist = (*g_ptr).getEdgeList() ;
	ok = (*temp_edgelist).getFirstNode(temp_edge1) ;
	while (ok) {
            (*temp_ipt_list).clear() ;
	    s_node1 = temp_edge1->getSNode() ;
	    e_node1 = temp_edge1->getENode() ;
            (*temp_edge1)._intersections((*s_node1).getPPos(), 
		(*s_node1).getType(), (*s_node1).getBoxSize(),
		(*e_node1).getPPos(), (*e_node1).getType(),
                (*e_node1).getBoxSize(), pos1, pos2) ;
	    ok2 = (*temp_edgelist).getFirstNode(temp_edge2) ;
	    while (ok2) {
	        s_node2 = temp_edge2->getSNode() ;
	        e_node2 = temp_edge2->getENode() ;
		if (s_node2 != s_node1 && s_node2 != e_node1 &&
		    e_node2 != s_node1 && e_node2 != e_node1) {
	  	    /*  no overlap of end points, 
		     *  calculate intersection points 
		     */
            	    (*temp_edge2)._intersections((*s_node2).getPPos(), 
		        (*s_node2).getType(), (*s_node2).getBoxSize(),
		        (*e_node2).getPPos(), (*e_node2).getType(),
                        (*e_node2).getBoxSize(), pos3, pos4) ;

                    if (lines_intersect(pos1, pos2, pos3, pos4, cur_t))
                        (*temp_ipt_list).insert(cur_t) ;
		    }
		ok2 = (*temp_edgelist).getNextNode(temp_edge2) ;
	    }

	    /* select the longest segment to put the label */
	    _find_largest_segment(pos1, pos2, temp_ipt_list,
                                  pos1, pos2) ;
            temp_info = temp_edge1->getAttr() ;
            temp_label = temp_edge1->getLabel() ;
            temp_size.w = CURFONTWIDTH * strlen(temp_label) ;
            temp_size.h = CURFONTHEIGHT ;
            _sw_segedgepos(pos1, pos2, temp_size, temp_info) ;

	    // Following section added by Jeff Nielsen
	    // to adjust edges that go in both directions
	    if (g_ptr->searchEdge(e_node1->getID(), s_node1->getID())!= NULL) {
	       // Means there's an edge in the other direction
	       
	       float slope;
	       if (pos1.x == pos2.x) slope = 100.0;
	       else slope = float(abs(pos1.y - pos2.y)) /
		  (abs(pos1.x - pos2.x));
		      
	       if (slope < 0.5) {	  // Horizontal line
		  if (pos1.x < pos2.x) {
		     temp_info->label_ppos.y -= 3;
		     if (pos1.y < pos2.y) temp_info->label_ppos.y -= 10;
		  }
		  else {
		     if (pos1.y > pos2.y) temp_info->label_ppos.y += 3;
		     else temp_info->label_ppos.y += 14;
		  }
	       }
	       else {			  // Vertical or Diagonal line
		  if (pos1.y > pos2.y)
		     temp_info->label_ppos.x -=4;
		  else {
		     temp_info->label_ppos.x +=13;
		     if (pos1.x < pos2.x) temp_info->label_ppos.y -= 8;
		     else if (pos1.x > pos2.x)
			temp_info->label_ppos.y += 10;
		     else temp_info->label_ppos.x += 3;
		  }
	       }
	    }
	    // End section added by Jeff Nielsen
	    
	    /* get next edge to process */
	    ok = (*temp_edgelist).getNextNode(temp_edge1, temp_edge3) ;
	    if (ok) 
		temp_edge1 = temp_edge3 ;
	}
	delete temp_ipt_list ;
	return TRUE ;
}

/***********************************************************
 * End of manual.c         		   	 	   *
 ***********************************************************/









