/***********************************************************
 * treelay.c --- a layout algorithm for rooted trees       *
 * Jun Yang, November 17, 1994 				   *
 ***********************************************************/

/*--------------- Header Files ---------------*/

#include <stdio.h>
#include <math.h>
#include "swan.h"
#include "laytree.h"

#define NUMVERTEX	128	
#define MIN_X_SEPARATION 10
#define STTOPDIST       10
#define TY_SEPARATION	40

//-------------------
//  Tree_layout -- the main function for rooted tree layout algorithm
//___________________

bool Tree_layout(BasicGraph* g_ptr)
{
	BasicLayoutComp		*temp_lc ;
	BasicLayoutComp		*root_lc ;
	LCNBLIST		*temp_lclist ;
	BasicNode		*root_node ;
	BOXSIZE			temp_size ;

	if (g_ptr == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    return FALSE ;
	}

	if ((*(*g_ptr).getNodeList()).getNodeCount() > NUMVERTEX)
	    return FALSE ;

	if ((*(*g_ptr).getNodeList()).getNodeCount() == 0)
	    return TRUE ;

	root_lc = (*g_ptr).getRootLC() ;
	temp_lclist = (*root_lc).getChLCList() ;
	if (!(*temp_lclist).getFirstNode(temp_lc))
	    return FALSE ;

	if ((*temp_lc).getType() != TREE)
	    return FALSE ;

	if ((root_node = (*temp_lc).getFNode()) == NULL)
	    return FALSE ;

	(*g_ptr).clearTravFlags() ;
	if (!_cal_tree_boxandpos(root_node, 0, 0, 
				 &temp_size.w, &temp_size.h))
	    return FALSE ;
	(*temp_lc).setBoxSize(temp_size) ;
	(*root_lc).setBoxSize(temp_size) ;
	return TRUE ; 
} /* end of Tree_layout */

bool _cal_tree_boxandpos(BasicNode* root, 
		    	    int start_x, int start_y, 
		    	    int* rightside, int* bottomside)
{
	int 		branchPosition ;
	int 		leftPosition ;
	int 		rightPosition ;
	int		centerPosition = start_x ;
	int		bottomPosition ;
	NODENBLIST	*nb_list ;
	bool		ok ;
	struct _nb_node	*temp_nb ;
	BOXSIZE		temp_box ;
	LOCATION	temp_pos ;
	BasicNode	*lastTemp ;
	int		label_len ;
	LABEL		temp_label ;

	if (root == (BasicNode*) NULL) {
	    *rightside = start_x ;
	    *bottomside = start_y ;
	}
	else { // place subtrees
	    // Ensure the nominal position of the node is right of any
	    // other node. 
	    (*root).setTravFlag() ;
	    temp_box = (*root).getBoxSize() ;
	    rightPosition = start_x + temp_box.w ;
	    bottomPosition = start_y + temp_box.h ;
	    branchPosition = start_x ;
	    nb_list = (*root).getNBList() ;
	    ok = (*nb_list).getFirstNode(temp_nb) ;
	    while (ok) {
		if (!(*(temp_nb->id)).isTraversed())
		    break ;
		ok = (*nb_list).getNextNode(temp_nb) ;
	    }
	    if (ok) {
	  	temp_label = (*(temp_nb->edge_ptr)).getLabel() ;
		if (temp_label == NULL)
		    label_len = 0 ;
		else 	
		    label_len = strlen(temp_label) * CURFONTWIDTH ;
		if ((*temp_nb->id).getWidth()/2 < label_len) 
		    branchPosition += label_len -
				     (*temp_nb->id).getWidth()/2 ;

	        _cal_tree_boxandpos(temp_nb->id, branchPosition, 
			start_y + temp_box.h + TY_SEPARATION +
			STTOPDIST,
			&rightPosition, &bottomPosition) ;

		leftPosition = ((*temp_nb->id).getRPos()).x ;
		lastTemp = temp_nb->id ;
		branchPosition = rightPosition + MIN_X_SEPARATION ;
		ok = (*nb_list).getNextNode(temp_nb) ; 
		while (ok) {
		    if (!(*(temp_nb->id)).isTraversed()) {
	  	    temp_label = (*(temp_nb->edge_ptr)).getLabel() ;
		    if (temp_label == NULL)
		        label_len = 0 ;
		    else 	
		        label_len = strlen(temp_label) * CURFONTWIDTH ;
		    if ((*temp_nb->id).getWidth()/2 < label_len) 
		    branchPosition += label_len -
				     (*temp_nb->id).getWidth()/2 ;
                    _cal_tree_boxandpos(temp_nb->id, branchPosition,
                          start_y + temp_box.h + TY_SEPARATION +
			  STTOPDIST,
                          &rightPosition, &bottomPosition) ;

		    lastTemp = temp_nb->id ;
		    branchPosition = rightPosition + MIN_X_SEPARATION ;
		    }
		    ok = (*nb_list).getNextNode(temp_nb) ;
		}
		centerPosition = (leftPosition+((*lastTemp).getRPos()).x)/2 ;
	    }
	
	    temp_pos.x = centerPosition ;
	    temp_pos.y = start_y ;
	    (*root).setRPos(temp_pos) ;
	    *rightside = rightPosition ;
	    if (start_y + temp_box.h > bottomPosition) 
		*bottomside = start_y + temp_box.h ;
	    else
		*bottomside = bottomPosition ;
		
	} 
	return TRUE ;
} /* _TREE_cal_boxandpos */

bool _TREE_nolabel_edge_ppath(BasicGraph* g_ptr)
{
        BasicEdge*      temp_edge ;
        BasicNode       *s_node, *e_node ;
        EDGELIST*       temp_edgelist ;
        bool         ok ;
        CURVE           *ppath ;
        BasicLayoutComp *lc1 ;
        LOCATION        spos, epos, pos1, pos2;
        BOXSIZE         ssize, esize ;
        Stack<SWPOINT>    temp_sk ;
        temp_edgelist = (*g_ptr).getEdgeList() ;
        ok = (*temp_edgelist).getFirstNode(temp_edge) ;
        while (ok) {
            s_node = (*temp_edge).getSNode() ;
            e_node = (*temp_edge).getENode() ;
            lc1 = (*s_node).getLayoutComp() ;

            spos = (*s_node).getPPos() ;
            epos = (*e_node).getPPos() ;
            ssize = ((*s_node).getAttr())->size ;
            esize = ((*e_node).getAttr())->size ;
            ppath = (*temp_edge).getPPath() ;
            (*ppath).clear() ;

	    if (spos != UnsetPos && epos != UnsetPos) {
            if ((*temp_edge)._intersections(spos,
                (*s_node).getType(), (*s_node).getBoxSize(),
                epos, (*e_node).getType(),
                (*e_node).getBoxSize(), pos1, pos2)) {
                (*ppath).insert(pos1) ;
                (*ppath).insert(pos2) ;
            }
	    }
            ok = (*temp_edgelist).getNextNode(temp_edge) ;
	}
	return TRUE ;
}

bool _TREE_labeled_edge_ppath(BasicNode *root)
{
	NODENBLIST	*nb_list ;
	bool		ok, nEven ;
	struct _nb_node	*temp_nb ;
	LOCATION	pos1, pos2, pos3, pos4 ;
	CURVE		*temp_path ;
	BasicNode	*nt ;
	int		n, i, half_pt, x_incr, cur_x ;

	if (root == (BasicNode*) NULL) 
	    return TRUE ;

	// Ensure the nominal position of the node is right of any
	// other node. 
	(*root).setTravFlag() ;
	nb_list = (*root).getNBList() ;
	n = (*nb_list).getNodeCount() ;
	switch (n) {
	case 0:
	    return TRUE ;
	
	case 1:     
	    ok = (*nb_list).getFirstNode(temp_nb) ;
	    if ((*(temp_nb->id)).isTraversed())
		return TRUE ;
	    temp_path = new CURVE ;
	    switch ((*root).getType()) {
	    case BOX:
                pos1.x = ((*root).getPPos()).x + (*root).getWidth()/2 ;
                pos1.y = ((*root).getPPos()).y + (*root).getHeight() ;
                pos2.x = ((*(temp_nb->id)).getPPos()).x + 
			 (*(temp_nb->id)).getWidth()/2 ;
                pos2.y = ((*(temp_nb->id)).getPPos()).y ;
                (*temp_path).insert(pos1) ;
                (*temp_path).insert(pos2) ;
	        (*(temp_nb->edge_ptr)).setPPath(temp_path) ;
		(*temp_path).clear() ;
                break ;
		
	    case CIRCLE:
		break ;

	    default:
	        break ;
	    }
	    delete temp_path ;
	    _TREE_labeled_edge_ppath(temp_nb->id) ;
	    return TRUE ;

	default:	// more than two neighbours
	    nt = NULL ;
	    ok = (*nb_list).getFirstNode(temp_nb) ;
	    while (ok) {
	        if ((*(temp_nb->id)).isTraversed()) {
		    nt = temp_nb->id ;
		    break ;
		}
		ok = (*nb_list).getNextNode(temp_nb) ;
	    }
	    if (nt != NULL) // the parent node has been traversed
		n-- ;
	    x_incr = (*root).getWidth() / (n+1) ;
	    if (n % 2 == 0) {
		nEven = TRUE ;
		half_pt = n/2 ;
	    }
	    else {
		nEven = FALSE ;
		half_pt = n/2 + 1 ;
	    }
	    cur_x = ((*root).getPPos()).x + x_incr ;
	    i = 1 ;
	    temp_path = new CURVE ;
	    ok = (*nb_list).getFirstNode(temp_nb) ;
	    while (ok) {
	        if (temp_nb->id != nt) {
		    pos1.x = cur_x ;
		    pos1.y = ((*root).getPPos()).y + (*root).getHeight() ;
		    (*temp_path).insert(pos1) ;
		    pos2.x = pos1.x ; 
		    if (i <= half_pt)	// left half
		        pos2.y = pos1.y + STTOPDIST - (half_pt - i)
							 * BUSDIST ;
		    else {
		        if (nEven)
			    pos2.y = pos1.y + STTOPDIST - (i-1-half_pt)
							* BUSDIST ;
			else 
			    pos2.y = pos1.y + STTOPDIST - (i-half_pt)
                                                        * BUSDIST ;
		    }
		    (*temp_path).insert(pos2) ;
		    pos3.x = ((*(temp_nb->id)).getPPos()).x + 
		             (*(temp_nb->id)).getWidth()/2 ;
		    pos3.y = pos2.y ;
		    (*temp_path).insert(pos3) ;
		    pos4.x = pos3.x ;
		    pos4.y = ((*(temp_nb->id)).getPPos()).y ;
		    (*temp_path).insert(pos4) ;
		    (*(temp_nb->edge_ptr)).setPPath(temp_path) ;
		    (*temp_path).clear() ;

		    // modify some increments
		    i++ ;
		    cur_x += x_incr ;
		    _TREE_labeled_edge_ppath(temp_nb->id) ;
		}
		ok = (*nb_list).getNextNode(temp_nb) ;
	    }	
	    break ;
	}
	delete temp_path ;
	return TRUE ;
}

/***********************************************************
 * End of treelay.c       				   *
 ***********************************************************/
